﻿;;;-*- Mode: Lisp; Package: "MOSS" -*-
;;;=================================================================================
;;;14/03/06
;;;		
;;;	               M O S S - L O A D - (File MOSS-LOAD.LiSP)
;;;		       Copyright Barthes@UTC 2002-2014
;;;	
;;;=================================================================================

;;; This file loads the MOSS system.
;;; When loaded with OMAS, does not open any specific window.
;;; When loaded as a stand alone application, loads the init menu.
#|
2007
 0528 rewriting file to have a single file for MCL and ACL
 0529 works fwith ACL
 0728 change load function to load patches from list and take care of compiler
      presence
 0728 make cl-user use moss
2008
 0604 === v5.11.2
 0610 adding the creation of the OMAS package useful when loading MOSS only
      removing #+OMAS, #+MOSS, etc.
 0621 === v7.0.0 number is now number of OMAS platform
 0705 getting the version number from the OMAS-MOSS folder name
2010
 1121 adding *moss-directory-pathname* as a synonym of *moss-directory-string*
2012
 1017 returning to own MOSS numbering with starting with version 8.0.0
2014
 0306 introducing UTF-8 files
2016
 0506 creating *context* and *version-graph* in cg-user package
|#

;;; create the MOSS package if loading for the first time
;;;(eval-when (:compile-toplevel :load-toplevel :execute)
;;;  (unless (find-package :moss) (make-package :moss))
;;;  ;; we must create the OMAS package for using the same compiled code with or
;;;  ;; without OMAS
;;;  (unless (find-package :omas) (make-package :omas)))

(defpackage  :moss (:use :cl))

(in-package :moss)

;;;==============================================================================
;;;                        Version number
;;;==============================================================================
;;; MOSS versionis maintained independently of OMAS versions. It is supposed to
;;; evolve less frequently than the OMAS functions

(defparameter *moss-version-number* "9.0")

;;;==============================================================================
;;;----------------------------------------------------------------- DFORMAT-RESET

(defun dformat-reset (tag)
  (setf (get tag :trace) nil)
  )

;;;------------------------------------------------------------------- DFORMAT-SET
;;; debugging function, needs to be in front

(defun dformat-set (tag level)
  (setf (get tag :trace) t)
  (setf (get tag :trace-level) level)
  )

;;; Kludge: define EDITOR class to keep compiler happy when calling save-old-value
;;; or  save-new-id

#-MCL
(defClass editor (dialog)
  ())

;;; Kludge: define an OMAS package to keep the compiler quiet when compiling some
;;; files. Note however that :OMAS is not put into *features*

(defpackage :omas)

;;; ------------------------------------------------------------------------------
;;; if no file was called before this one, then we must get the version info
;;; they are in the OMAS-MOSS directory one level up the file tree

(unless (member :moss *features*)
  (let ((init-file 
         (make-pathname
          :device (pathname-device *load-pathname*)
          :host (pathname-host *load-pathname*)
          :directory (pathname-directory *load-pathname*)
          :name "MOSS-version-info"
          :type "lisp")))
    (print init-file)
    (load init-file :external-format :utf-8))
  ;; record moss at system level
  (pushnew :moss *features*)
  )

;;; ------------------------------------------------------------------------------

;;; record loading path at loading time
(defParameter *moss-directory-string*
  (make-pathname :name nil :type nil :defaults *load-pathname*))

(defParameter *moss-directory-pathname*
  (make-pathname :name nil :type nil :defaults *load-pathname*))

;(print *moss-directory-string*)

;;; cook up patches file name 
(defParameter *moss-patches-file* 
  (make-pathname
   :device (pathname-device *moss-directory-string*)
   :host (pathname-host *moss-directory-string*)
   :directory (append (pathname-directory *moss-directory-string*)
                      (list "MOSS source"))
   :name (format nil "~A-patches" *moss-prefix*)
   :type "lisp"))

;(print *moss-patches-file*)

;;; ------------------------------------------------------------------------------
;;; build the list of file access paths from the file names and directory path

(defParameter *moss-files*
  (mapcar #'(lambda (xx)
              (make-pathname
               :device (pathname-device *moss-directory-string*)
               :host (pathname-host *moss-directory-string*)
               :directory (append (pathname-directory *moss-directory-string*)
                                  (list "MOSS source"))
               :name (format nil "~A-~A" *moss-prefix* xx)))
          *moss-file-names*)
  "MOSS file pathnames")

;;;------------------------------------------------------------- COMPILE-IF-NEEDED
;;;
(Defun Compile-If-Needed (Source Compiled)
  "Compiles Source If Not Compiled, If Changed.
Arguments:
   Source: File Pathname of Source Code
   Compiled: File Pathname of Compiled Result
Result:
   Unimportant"
  (Let ()
    (When (or (Not (Probe-File Compiled))
              (< (File-Write-Date Compiled)
                 (File-Write-Date Source)))
      (Compile-File Source :Output-File Compiled :Verbose T
            :External-Format :utf-8))))

;;;--------------------------------------------------------------------- LOAD-MOSS
;;; the following lines load MOSS as follows:
;;;   - first compute the paths for source and compiled files
;;;   - if source is there, then check if different from the compiled version
;;;     - if different, then recompile the source file
;;;     - then load the compiled file

(defUn load-moss ()
  "load MOSS as follows:
   - first compute the paths for the source and compiled files
   - if source is there, then check if different from the compiled version and if 
     the compiler is available:
     - if different or no compiled file, compile the source
     - then load the compiled file"
  (declare (special *moss-file-names* *moss-prefix*))
  (let ()
    (format t "~&;*** --- Loading MOSS --- ***")
    (dolist (filename *moss-file-names*)
      (load-moss-file (format nil "~A-~A" *moss-prefix* filename)))
    :MOSS-files-loaded))

;;;---------------------------------------------------------------- LOAD-MOSS-FILE
;;; here we forget about MCL

(defun load-moss-file (filename)
  "given a string specifying the omas file, loads it compiling it if needed. Uses ~
   *moss-directory-string* to locate the directory containing the file.
Argument:
   filename: a string
Return:
   :done"
  (declare (special *moss-directory-string* *features* *moss-prefix*))
  (let ((source 
         (make-pathname
          :device (pathname-device *moss-directory-string*)
          :host (pathname-host *moss-directory-string*)
          :directory (append (pathname-directory *moss-directory-string*)
                             (list "moss source"))
          :name filename
          :type "lisp"
          ))
        (compiled 
         (cond
          ((member :OMAS *features*)
           ;; compile folder for OMAS/MOSS
           (make-pathname
            :device (pathname-device *moss-directory-string*)
            :host (pathname-host *moss-directory-string*)
            :directory (append (pathname-directory *moss-directory-string*)
                               (list "MOSS source")
                               (list "FASL"))
            :name filename
            :type "fasl"))
          (t
           ;; compile folder for MOSS stand alone
           (make-pathname
            :device (pathname-device *moss-directory-string*)
            :host (pathname-host *moss-directory-string*)
            :directory (append (pathname-directory *moss-directory-string*)
                               (list "MOSS source")
                               (list "MFASL"))
            :name filename
            :type "fasl")
           )))
        )
    
    #+compiler ; when compiler is there, compile file if needed
    (cond
     ;;=== if source file is there, check for possible recompiling
     ;; since patches are available as source file, they will be recompiled
     ;; if needed
     ((probe-file source)
      (compile-if-needed source compiled)
      ;(format *debug-io* "~&Load-pathname 1: ~S" compiled)
      (load compiled :external-format :utf-8))
     ;;=== if only compiled file is there load it
     ((probe-file compiled)
      ;(format *debug-io* "~&Load-pathname 2: ~S" compiled)
      (load compiled :external-format :utf-8))
     ;;=== otherwise complain
     (t (warn "~&File missing: ~S" filename)))
    
    ;; when compiler is not there (some distributed versions), load source
    ;; file if more recent than compiled file
    #-compiler
    (cond
     ;;=== if source file is available, and more recent than compiled file, load it
     ((and (probe-file source) (probe-file compiled)
           (<= (file-write-date compiled)
               (file-write-date source)))
      (load source :external-format :utf-8)
      )
     ;;=== if source file is available but not compiled file, then load it
     ((probe-file source)
      (load source :external-format :utf-8))
     ;;=== if source not there, check compiled and load it
     ((probe-file compiled)
      (load compiled :external-format :utf-8))
     ;;=== no source nor compiled file there
     (t (warn "~&File missing: ~S" filename)))
    :file-loaded))

;;; ------------------------------------------------------------------------------
;;;

(load-moss)

(provide :MOSS)

;;; tell compiler that CG-USER inherits exported symbols from MOSS
(eval-when (:compile-toplevel :load-toplevel :execute)
  (in-package :cg-user)
  (use-package :moss)
  (in-package :moss))

(eval-when (:load-toplevel)
  ;; create a stub in the user environment so that we can work and create items
  ;; in it directly
  (%create-new-package-environment :cg-user "CG-USER")
  ;;; launch MOSS by opening MOSS window
  (unless (member :omas *features*) ; JPB0806
    (cg::set-foreground-window (make-moss-init-window)))
  )

;;;============================ Function to get references ========================

;;; the following function traces in what function a specific symbol appears
;;; by examining the code of the function in the corresponding file.

(defUn mref (symbol &optional show-file)
  "traces in what function a specific symbol appears by examining the code of the ~
   function in the corresponding file."
  (let (expr)
    (dolist (filename *moss-files*)
      (with-open-file 
        (ss (merge-pathnames ".lisp" filename))
        ;; if wanted, we print the path of the file we are checking
        (when show-file
          (print (merge-pathnames ".lisp" filename)))
        (loop
          (setq expr (read ss  nil :end))
          (when (eql :end expr) (return :done))
          (when (and (listp expr)(member (car expr)'(defUn macro defmethod))
                     (mappears-in? symbol expr))
            (print (cadr expr))))))
    :done))

(defUn mappears-in? (symbol expr)
  (cond ((null expr) nil)
        ((equal symbol expr) t)
        ((listp expr)
         (or (mappears-in? symbol (car expr))
             (mappears-in? symbol (cdr expr))))))


(format t "~%;*** MOSS version ~A loaded ***" moss::*moss-version-number*)
(format t
        "~%;*** Methods are cached: *cache-methods* set to T *** ~
         ~&;*** Fancy inheritance turned off *lexicographic-inheritance* set to T ***")
