﻿;;;-*- mode: lisp; package: "cscwd-web" -*-
;;;===============================================================================
;;;14/11/12
;;;                    AGENT CSCWD-WEB (file: CSCWD-WEB.lisp)
;;;                    Copyright barthès@utc, 2014
;;; 
;;;===============================================================================
;;; this agent is a go between between the web server and various PAs. it receives 
;;; messages, select the right PA and forwards the message. it then transfers the
;;; answer to the server thread.
;;;
;;; CSCWD-WEB has the following skills:
;;;   - GET-LANGUAGE to get the PA language
;;;   - LOCATE-PA to locate the PA and obtain its IP
;;;   - LOG-ERROR logs error into th HDSRI-LOG.txt file
;;;   - PROCESS-WEB-REQUEST more general than the following
;;;   - PROCESS-WEB-TEXT send text to the PA to be processed
;;;   - SHUT-DOWN-SERVER unused
;;;   - START-SERVER starts Allegroserve and post pages
;;;
;;; CSCWD-WEB handles the following pages (Allegroserve entities):
;;;   - BUG REPORT FORM
;;;   - AUTHOR PAGE (pop up to display author names)
;;;   - ERROR PAGE
;;;   - INDEX the driving page of the application
;;;   - LOGIN the application login page
;;;   - PASSWORD FORM to change password
;;;   - PERSON PAGE pop up to display persons
;;;   - WEB DIALOG FORM for hosting dialogs
;;;   - CONCEPT queries (see /concept entity)
;;;
;;; CSCWD-WEB indirectly requires the following skills:
;;;   - :authorized? (M)
;;;   - :check-password (M)
;;;   - :get-all-persons (P)
;;;   - :get-language to get the language of a target agent
;;;   - :get-member-description (M)
;;;   - :locate-pa (W)
;;;   - :log-error (W)
;;;   - :process-web-text (P)
;;;   - :record-person (Staff, P)
;;;   - :set-password (M)
;;;   - :start-server (W)


;;; CSCWD-WEB is a service agent, thus, before sending messages, it should create a
;;; task, which sets up the picture for receiving the answer.

#|

2014
 0916 Creation
 0930 Utility functions loading, static-start-server (added static files and publish-concept-resource)
      publish-concept-resource: new function, new links in add-index-form (mfk)
 1027 adding "entry-language" parm to the index page for transferring to the locate
      page, so that we can retrieve objects using any language
 1112 modifying index to introduce role prototypes
      adding pop up window
|#

(defpackage :CSCWD-WEB (:use :cl :moss :omas :excl :net.aserve :net.html.generator)
  (:nicknames :web))
(in-package :cscwd-web)

;;; ===============================================================================
;;;
;;;              Temporary loading utility functions for rendering (mfk)
;;;
;;; ===============================================================================
(omas::compile-and/or-load 
 (merge-pathnames "utils.lisp" 
                  (string+ (namestring omas::*omas-directory-string*) 
                           "applications\\nb-ontocodesign\\")))


;;;===============================================================================
;;; 
;;;                           creating web agent
;;;
;;;===============================================================================

(omas::defagent :cscwd-web)

;;;===============================================================================
;;; 
;;;                     globals, service macros and functions 
;;;
;;;===============================================================================

;;;==============================================================================
;;;                                GLOBALS
;;;==============================================================================

;;; when using the web server the text is only part of the web page...

(defparameter *web-server-port* 80 "server port for serving ONTOCODESIGN forms.")

;;; multilingual tags
;;; NOTE:
;;; Currently we use only French tags : a problem when specifying language when
;;; giving the name of the PA is that, in order to create the PA with the right
;;; language, requires files like XXXXX-FR and corresponding ontology and dialog
;;; files. Because we do not use natural language to access data, we do not need 
;;; that and can clone PA files from the XXXXX-EN one, e.g. BARTHES-EN
;;; Changing the language afterwards does not pose any particular problem.

(defmacro defmln (name expr)
  "transforms the old string format into the new list format"
  `(defparameter ,name (mln::make-mln ,expr)))

(defmln *ir-add/edit-concept*
    '(:fr "Créer ou éditer un concept" :en "Add or edit a concept"))
(defmln *ir-add/edit-link*
    '(:fr "Créer ou éditer un rôle ou lien générique" 
          :en "Add or edit a role or link prototype"))
(defmln *ir-beta-test*
    '(:fr "Pendant la phase de bêta test :" :en "During bêta tests:"))
(defmln *ir-assistant*
    '(:fr "Nom de votre assistant si différent du vôtre :" 
          :en "Your personal assistant name if different from yours:"))
(defmln *ir-comments*
    '(:fr "Commentaires" :en "Comments"))
(defmln *ir-comments-go*
    '(:fr "Commentaires ou rapports d'incidents" :en "Comments or Bugs"))
(defmln *ir-concept*
    '(:fr "Concept" :en "Concept"))
(defmln *ir-create-role*
    '(:fr "Créer un nouveau lien (role)" :en "Create a new link (role)"))
(defmln *ir-dialog-answer*
    '(:fr "Réponse à : " :en "Answering: "))
(defmln *ir-dialog-explain*
    `(:fr ,(format nil "Le dialogue est flexible mais n'est pas ouvert. Votre ~
            assistant se concentre sur les éléments d'information concernant les ~
            concepts ou les roles.")
          :en ,(format nil "Dialog is flexible but not open. The PA is concentrating ~
            on handling information about concepts or roles.")))
(defmln *ir-dialog-question*
    '(:fr "Question ?" :en "Question?"))
(defmln *ir-display*
    '(:fr "Affichage de l'ontologie" :en "Display of ontology"))
(defmln *ir-display-graph*
    '(:fr "Affichage comme un graphe" :en "Display as a graph"))
(defmln *ir-display-tree*
    '(:fr "Affichage sous forme d'arbre" :en "Display as a tree"))
(defmln *ir-edit-concept*
    '(:fr "Editer un concept existant" :en "Edit an existing concept"))
(defmln *ir-edit-link*
    '(:fr "Editer un lien existant" :en "edit an existing link"))
(defmln *ir-error-text* 
    '(:fr "Merci d'écrire vos remarques ou de signaler les dysfonctionnements ci-contre."
          :en "Please signal and/or describe the problem."))
(defmln *ir-dialog*
    '(:fr "Consultation des informations"
          :en "Consulting the database"))
(defmln *ir-features*
    '(:fr "Options :" :en "Features:"))
(defmln *ir-goto-dialog*
    '(:fr "Activer le dialogue" :en "Activate dialog"))
(defmln *ir-link*
    '(:fr "Rôle/Lien générique" :en "Role/Link prototype"))
(defmln *ir-login-error*
    `(:fr ,(format nil "Si vous ne pouvez vous connecter, il est possible que vous ~
             n'ayez pas d'assistant personnel. Merci de contacter JP. Barthès.")
          :en ,(format nil "If this does not work, you may  not have a personal ~
                 assistant, please contact JP. Barthès.")))
(defmln *ir-login-explain*
    `(:fr ,(format nil "Le login est utilisé pour activer votre assistant personnel ~
           ou vous en créer un nouveau, ce qui peut prendre quelques secondes.")
          :en ,(format nil "Login is used to locate your personal assistant or to ~
            create a new one, which can take a few seconds.")))
;;;(defmln *ir-login-language*
;;;    '(:fr "Quelle langue ? ex. FR, EN,... (défaut EN)"
;;;          :en "What language? e.g. EN, FR,...(default EN)"))	  
(defmln *ir-login-language*
    '(:en "Language currently is set to English;"))
(defmln *ir-misc*
    '(:fr "Divers :" :en "Miscellaneous:"))
(defmln *ir-new-pass*
    '(:fr "Nouveau mot de passe :" :en "New password:"))
(defmln *ir-new-pass-check*
    '(:fr "Nouveau mot de passe (vérification) :" :en "New password (check):"))
(defmln *ir-no-assistant*
    '(:fr "Je ne trouve pas votre assistant. merci de vérifier l'orthographe."
          :en "I cannot find your personal assistant, please check spelling."))
(defmln *ir-old-pass*
    '(:fr "Ancien mot de passe :" :en "Old password:"))
(defmln *ir-password*
    '(:fr "Modifier le mot de passe" :en "Changing the password"))
(defmln *ir-password-go*
    '(:fr "Mot de passe" :en "Password"))
(defmln *ir-password-no-check*
    '(:fr "Le nouveau mot de passe n'est pas celui retapé."
          :en "New password and verification do not check."))
(defmln *ir-password-no-user*
    '(:fr "Je n'arrive pas à récupérer le nom de l'utilisteur."
          :en "I can't recover the username."))
(defmln *ir-password-not-ok*
    '(:fr "Échec de la mise à jour du mot de passe."
          :en "Password could not be changed."))
(defmln *ir-password-ok*
    '(:fr "Mot de passe modifié." :en "Password has been changed."))
(defmln *ir-password-wrong*
    '(:fr "Ancien mot de passe erronné." :en "Wrong password."))
(defmln *ir-report-title*
    '(:fr "IEEE/SMC Comité CSCWD - Remarques ou erreurs"
          :en "IEEE/SMC CSCWD Technical Committee - Remarks or Bugs"))
(defmln *ir-report-title-annotations*
    '(:fr "IEEE/SMC Comité CSCWD - Annotations"
          :en "IEEE/SMC CSCWD Technical Committee - Annotations"))
(defmln *ir-return* 
    '(:fr "Retour vers la page d'accueil" :en "Home"))
(defmln *ir-send* 
    '(:fr "Envoyer" :en "Send"))
(defmln *ir-text*
    '(:fr "texte" :en "text"))
(defmln *ir-title* 
    '(:fr "IEEE/SMC Comité CSCWD" :en "IEEE/SMC CSCWD Technical Committee"))
(defmln *ir-title-dialog*
    '(:fr "IEEE/SMC Comité CSCWD - Dialogue"
          :en "IEEE/SMC CSCWD Technical Committee - Dialog"))
(defmln *ir-title-pass*
    '(:fr "NEWS: Nouvelles internationales - Changement du mot de passe"
          :en "International News - Password Reset"))
(defmln *ir-username*
    '(:fr "Votre login :" :en "User login name:"))

;;;------------------------------------------------------------------ SELECT-NAME

(defun select-name (input)
  (mln::extract-to-string input :always t))

#|
(select-name *ir-username*)
"User login name:"
(setq *language* :FR)
(select-name *ir-username*)
"Votre login :"
(setq *language* :it)
(select-name *ir-username*)
"User login name:"
|#

;;;==============================================================================
;;;                             SERVICE MACROS
;;;==============================================================================

;;; macro to insert as the title of all cscwd pages

(defmacro cscwd-title (title)
  ``((:table :border "0" :cols "2" :width "100%")
     (:tbody
      ;;=== dialogue
      (:tr 
       ((:td :width "10%") 
        "<IMG style=\"width:130px; height:100px\"
                  SRC=\"http://www.utc.fr/~barthes/cscwd-logo.gif\">")
       (:td (:h2 :center ,(mln::extract-to-string ,title :always t)))
       )
      )))

;;;==============================================================================
;;;                            SERVICE FUNCTIONS
;;;==============================================================================

;;;---------------------------------------------------- CREATE-ASSISTANT-FOR-CSCWD
;;; This function creates and installs a temporary PA agent, cloning the
;;; necessary file, but not creating new files in the application folder. I.e.,
;;; once the PA is created the corresponding temporary files are deleted
;;; the assistant stub should be called, e.g., XXXXX-FR
;;; user-description is something like:
;;;  ("personne" ("nom" "Charara") ("prénom" "Ali")
;;;   ("assistant personnel" "Nestor"))

(defun create-assistant-for-cscwd (agent-key &key (language :EN) user-description)
  "creates a new assistant by cloning ASSISTANT-STUB according to ~
   language, e.g. XXXXX-FR for French.
Arguments:
   key: key of the future PA
   language (key): language of the PA (default is English)
   user-description (key): description of the user, master of the PA
Return:
   the key of the new assistant or nil in case of problem."
  (let* ((assistant-ref (symbol-name agent-key))
         (in-filename (format nil "XXXXX-~A" language))
         assistant-pathname)
    (format t "~2%;---------- Entering create-assistant-for-cscwd") 
    
    ;; check first if agent alread exists (e.g. when the function is called again)
    (when (assoc agent-key (omas::local-agents *omas*))
      (warn "agent ~S already exists" agent-key)
      (return-from create-assistant-for-cscwd agent-key))
    
    ;; first clone the PA stub, e.g. XXXXX-FR.lisp, substituting master-name to MMMMM
    (setq assistant-pathname
          (omas::clone-and-adjust-file 
           assistant-ref in-filename 
           :substitutions `(("YYYYY" . ,(format nil "~S" user-description)))))
    (unless assistant-pathname
      (return-from create-assistant-for-cscwd nil))
    
    ;;== OK we can load everything
    (omas::load-agent :agent-file-pathname assistant-pathname)
    
    ;;== delete temporary files
    (delete-file assistant-pathname)
    
    ;;== return success
    agent-key))

;;;--------------------------------------------------------------------- get-pa-ip

(defun get-pa-ip (req)
  "from a web request determines who is the sender, recording the ip of the sending
   machine. we assume different machines for different senders. another solution ~
   would be to use cookies.
argument:
   req: the web request from the webserver thread
return:
   the ip of the pa."
  (let ((socket (request-socket req))
        ip)
    ;(format t "~%; get-pa-ip /socket:~%;  ~s" socket)
    ;; extracts the ip of the calling machine
    (setq ip (socket:remote-host socket))
    (format t "~%; get-pa-ip /remote-host: ~s: ~s~%" ip (socket:ipaddr-to-dotted ip))
    ;; check for an entry in the users slot to get the pa key
    ip))

;;;-------------------------------------------------------------------- get-pa-key

(defun get-pa-key (req)
  "from a web request determines who is the sender, recording the ip of the sending ~
   machine. we assume different machines for different senders. another solution ~
   would be to use cookies.
argument:
   req: the web request from the webserver thread
return:
   the ip of the pa."
  ;;;  (format t "~%; get-pa-keomas::omas-application-directoryy /calling get-pa-ip")
  ;;;  (setq ip (get-pa-ip req))
  ;; check for an entry in the users slot to get the pa key
  (let ((ip (socket:remote-host (request-socket req))))
    (format t "~%; get-pa-key /users: ~s" (omas::users *omas*))
    (intern (omas.web::wu-get ip "pa-key") :keyword)
    ))

;;;------------------------------------------------------- remove-empty-parameters

(defun remove-empty-parameters (alist)
  "takes an a-list and removes pairs whose cdr is an empty string"
  (remove nil
          (mapcar #'(lambda(xx) (unless (equal "" (cdr xx)) xx)) alist)))

#|
(remove-empty-parameters '(("pays" . "japon") ("correspondant" . "inoue")
                           ("prénom correspondant" . "")
                           ("initiale correspondant" . "t") ("url" . "")
                           ("partenaire" . "u of tsukuba") ("sigle partenaire" . "")
                           ("correspondant utc" . "barthès") ("type de contact" . "r")
                           ("date de début" . "2012")))
(("pays" . "japon") ("correspondant" . "inoue") ("initiale correspondant" . "t") 
 ("partenaire" . "u of tsukuba")("correspondant utc" . "barthès")
 ("type de contact" . "r") ("date de début" . "2012"))
|#

;;;-------------------------------------------------------------------------- row

(defun row (&key header name (size 100) read-only value (type "text") 
                 &allow-other-keys)
  "produces a single row in an html table.
arguments:
   header (key): name of the field
   name (key): name of the html parameter
   size (key): width of the value field (default 100)
   value (key): initial value
   read-only (key): if true declares the field to be read-only (does not work)
return:
   a string to be inserted into an html macro"
  (moss::string+ 
   "<tr><td>" header "</td>" "<td><input name=" name  " type=" type
   " size=" size
   (if value (format nil " value=~s" value) "")
   (if read-only " readonly>" ">")
   "</td></tr>"))

#|
(row :header "nom" :name "name" :read-only t)
"<tr><td>nom</td><td><input name=name type=text size=100 readonly></td></tr>"
(row :header "nom" :name "name")
"<tr><td>nom</td><td><input name=name type=text size=100></td></tr>"
|#

;;;===============================================================================
;;;===============================================================================
;;;                                 SKILLS 
;;;===============================================================================
;;;===============================================================================

;;;========================================================================= skill
;;;                               :GET-LANGUAGE
;;;===============================================================================
;;; we need this skill here to ask pa what language it understand. we callont send
;;; a message to pa directly since the server worker is not an agent

;;;========================================================================= skill
;;;                               :LOCATE-PA
;;;===============================================================================

(defskill :locate-pa :CSCWD-WEB
  :static-fcn static-locate-pa
  :dynamic-fcn dynamic-locate-pa
  :timeout-handler timeout-locate-pa)

(defun static-locate-pa (agent message args)
  "tries to locate a pa by sending an hello message with a timeout. if there is ~
   no answer will return failure to the caller."
  (declare (ignore message))
  (let ((key (cdr (assoc :data args)))
        (index (cdr (assoc :gate args)))
        
        file-pathname
        )
    ;; check first if agent is one of the local agents
    (when (assoc key (omas::local-agents *omas*))
      (omas::web-push-answer index t)
      (omas::web-open-gate index)
      (return-from static-locate-pa (static-exit agent :ok)))
    
    ;; is assistant in our application folder, if so load it and quit
    (when
        (probe-file 
         (setq file-pathname
               (merge-pathnames (omas::omas-application-directory *omas*)
                                (format nil "~a.lisp" key))))
      (omas::load-agent :agent-file-pathname file-pathname)
      (omas::web-push-answer index t)
      (omas::web-open-gate index)
      (return-from static-locate-pa (static-exit agent :loaded)))
    
    ;; otherwise send message to try to find it elsewhere, timeout is 5 seconds
    (env-set agent index :gate)
    (send-subtask agent :to key :action :hello :timeout 5)
    (static-exit agent :done)))

(defun dynamic-locate-pa (agent message answer)
  (declare (ignore message))
  (let ((index (env-get agent :gate)))
    ;; if we come here, then we have an answer
    (omas::web-push-answer index answer)
    (format t "~%; dynamic-locate-pa /answer-list: ~s" 
      omas.web::*web-answer-list*)
    (omas::web-open-gate index)
    (dynamic-exit agent t)))

(defun timeout-locate-pa (agent message)
  (declare (ignore agent message))
  ;; asks OMAS to kill the task, avoiding to resend messages
  :kill-task)

;;;========================================================================= skill
;;;                            :LOG-ERROR
;;;===============================================================================
;;; log errors into the hdsri-error-log.text file, creating it if necessary

(defskill :log-error :cscwd-web
  :static-fcn static-log-error)

(defun static-log-error (agent message args)
  "log errors into the cscwd-error-log.txt file."
  (declare (ignore message))
  (let ((text (cdr (assoc :data args)))
        (file-pathname            
         (make-pathname 
          :name "CSCWD-ERROR-LOG"
          :type "txt"
          :defaults (omas::omas-application-directory *omas*))))
    (format t "~%;--- file-pathname: ~S" file-pathname)
    (with-open-file (out file-pathname 
                         :direction :output 
                         :if-exists :append
                         :if-does-not-exist :create
                         :external-format :utf-8)
      (format out "~%~s~%~a~%" 
        (omas::date-string (get-universal-time)) text)))
  (static-exit agent :done))

;;;========================================================================= skill
;;;                            :PROCESS-WEB-REQUEST
;;;===============================================================================
;;; receives an internal message that must be sent to the destination agent. The
;;; origin of the message is usually one of the server threads.

(defskill :process-web-request :cscwd-web
  :static-fcn static-process-web-request
  :dynamic-fcn dynamic-process-web-request)

(defun static-process-web-request (agent message args)
  "function processes the data received by the web server to send the request to the
   target service agent.
arguments:
   agent: this agent
   message: incoming message
   args: ((:data ...)(:gate ...)(:to-agent ...)
     - data contains the text to be processed
     - gate contains the index of a synchronization variable. the function waiting
       for the answer is waiting on the gate associated with this index.
     - to-agent is the agent key of the destination pa
return:
   :done"
  (declare (ignore message))
  (let ((to-agent (cdr (assoc :to-agent args)))
        (action (cdr (assoc :action args)))
        (data (cdr (assoc :data args)))
        (language (cdr (assoc :language args)))
        (pattern (cdr (assoc :pattern args)))
        (gate (cdr (assoc :gate args)))
        (timeout (cdr (assoc :timeout args))))
    (env-set agent gate :gate)
    (send-subtask agent :to to-agent :action action 
                  :args `(((:data . ,data)
                           ,@(if language `((:language . ,language)))
                           ,@(if pattern `((:pattern . ,pattern)))
                           (:timeout . ,timeout)))
                  :timeout timeout  ; to kill process if it waits too long
                  )
    (static-exit agent :done)))

(defun dynamic-process-web-request (agent message answer)
  ;(format t "~%;web/ dynamic-process-web-request /message: ~s" message)
  ;; recover gate index
  (let ((index (env-get agent :gate))
        reason)
    (format t "~2%;<<< ~A: dynamic-process-web-request" (omas::name agent) )
    (format t "~%;---answer:~%  ~S" answer)
    
    ;(omas::message-print message)
    ;; if answer is a failure, then extract reason if any
    ;; a NIL answer is not considered a failure...
    (when (and answer (moss::is-answer-failure? answer))
      (setq reason (omas::error-contents message))
      (format t "~%;--- reason: ~S" reason)
      ;; build a list starting with :error followd by reason (a string)
      (setq answer (list :error reason))
      )
    ;; save answer into list of answers indexed by gate number
    (omas::web-push-answer index answer)
    ;(format t "~%;web/ dynamic-process-web-request /answer-list: ~s" 
    ;  omas.web::*web-answer-list*)
    ;; open gate
    (omas::web-open-gate index)
    (dynamic-exit agent :done)))

;;;======================================================================== skill
;;;                              :START-SERVER
;;;==============================================================================
;;; this enables the server and loads the entities for answering questions and
;;; filling the forms through web pages
;;; use connect instead, since connect published the needed pages

(defskill :start-server :cscwd-web
  :static-fcn static-start-server)

(defun static-start-server (agent message)
  "starts the http server, and sets up receiving pages for handling the requests"
  (declare (ignore message)
           (special *web-server-port*))
  (let ()
    ;; first start server
    (omas::start-web-server agent *web-server-port*)
    ;; publish a few pages
    (publish-annotations agent)
    (publish-bug-report-form agent)
    (publish-index agent)
    (publish-login-form agent)
    (publish-omas-web-edit-forms agent) ; web editor
    (publish-password-form agent)
    (publish-pop-up-concepts agent)
    (publish-web-dialog agent)   
    (publish-concept-resource agent)
    (publish-directory :prefix "/html" 
                       :destination (string+ (namestring omas::*omas-directory-string*)
                                             "applications\\nb-ontocodesign\\webres"))    
    (static-exit agent :done)))

;;;==============================================================================
;;;==============================================================================
;;;                                WEB PAGES
;;;==============================================================================
;;;==============================================================================

;;; checks can be done using the html parser
;;;   (require :phtml)
;;;   (use-package :net.html.parser)
;;;   (pprint (parse-html "..."))

;;;==============================================================================
;;;                          ANNOTATION FORM
;;;==============================================================================
;;; the formaction option in the HTML syntax does not seem to support parameters
;;; this we use an intermediate page to call the edit pages directly

(defun annotation-form (ip annotations)
  (format t "~%;--- anotation form... ip=~s" ip)
  (let ((agent-ref (omas.web::wu-get ip "agent-ref"))
        (ref (omas.web::wu-get ip "ref"))
        )
    (eval
     `(html
       (:html
        (:head 
         (:title ,(select-name *ir-report-title-annotations*))
         ((:meta :charset "utf-8")))
        (:body 
         "<font face=\"arial\">"
         "<fieldset style=\"background-color:yellow;\">"
         ,(cscwd-title *ir-report-title-annotations*)
         :br
         ((:table :border "0" :cols "2" :width "100%")
          (:tbody
           ;;=== show annotations
           (:tr
            ((:td :width "30%") "Annotations")
            (:td 
             ((:textarea :rows 10 :cols "100%" :read-only)
              (:princ ,annotations )))
            )
           :br
           (:tr
            (:td)
            (:td
             ((:a :href 
                  ,(string+ "editentry?agent-ref=" agent-ref 
                            "&owner=" ref
                            "&language=EN"
                            "&class-ref=annotation"
                            "&caller=index"
                            ))
              "Add annotation")
             ))
           ))
         :br :br
         ((:a :href "index") ,(select-name *ir-return*))
         ))))))

(defun publish-annotations (agent)
  (publish 
   :path "/annotations"
   :content-type "text/html"
   :function
   #'(lambda (req ent)
       ;(format t "~%;++++ publish-annotation-page /req: ~S~%~%" req)
       ;(format t "~%;++++ publish-annotation-page /(request-query req): ~S~%~%" 
       (with-http-response (req ent)
         (with-http-body (req ent)
           (publish-annotations-action agent req ent))))))

(defun publish-annotations-action (agent req ent)
  "functions associated to the entity handling the webdialog page"
  (declare (ignore ent))
  (format t "~2%;========== ~A: Entering publish-annotations-action"
    (omas::name agent))
  
  (let* ((parms (request-query req))
         (ip (read-from-string (get-field "ip" parms) nil nil))
         (ref (omas.web::wu-get ip "ref"))
         answer)
    (format t "~%;-- parms:~% ~S" parms)
    (format t "~%;-- ip:~% ~S" ip)
    (format t "~%;-- ref:~% ~S" ref)
    (format t "~%;-- user-entry:~% ~S" (omas.web::wu-get-user-entry ip))
    
    ;; get annotation summaries: an HTML string to be displayed in a text area
    (setq answer
          (omas::get-answer
           agent 
           ref
           :to-agent :CSCWD-ONTO
           :action :get-annotations
           ;:pattern '("annotation" ("author")("creation date"))
           :timeout 5
           :allow-nil-answer t))
    ;; in case of error or timeout we must return nil
    (format t "~%;--- answer ~S"  answer)
    
    (annotation-form ip answer)))

;;;==============================================================================
;;;                          BUG REPORT FORM
;;;==============================================================================
;;; to let the user report bugs

(defun bug-report-form (ip)
  (declare (ignore ip))
  (eval
   `(html 
     (:html
      (:head 
       (:title (:princ ,(select-name *ir-report-title*)))
       ((:meta :charset "utf-8")))
      (:body 
       "<font face=\"arial\">"
       ((:form :action "bugreportform")
        "<fieldset style=\"background-color:yellow;\">"
        ,(cscwd-title *ir-report-title*)
        :br
        (:princ ,(select-name *ir-error-text*))
        :br :br
        (:center ((:textarea :name "text" :rows 10 :cols "80")))
        :br
        (:center ((:input :type "submit" :value ,(select-name *ir-send*))))
        :br
        :br
        ((:a :href "index") (:princ ,(select-name *ir-return*)))
        ))))))

(defun publish-bug-report-form (agent)
  "post a form to let users report errors."
  (publish 
   :path "/bugreportform"
   :content-type "text/html"
   :function
   #'(lambda (req ent)
       (with-http-response (req ent)
         (with-http-body (req ent)
           (format t "~%; publish-bug-report-form /query-req:~%;  ~s"
             (request-query req))
           (publish-bug-report-form-action agent req ent))))))

;;;---------------------------------------------- publish-bug-report-form-action
;;; called by one of the server workers

(defun publish-bug-report-form-action (agent req ent)
  (declare (ignore ent)(special *language*))
  (let* ((query-req (request-query req))
         (ip (get-pa-ip req))
         (language (get-field "language" query-req))
         (*language* (omas::set-language language :only-return t))
         (text (get-field+ *ir-text* query-req)))
    (cond
     ;; first time around send bug report page
     ((null text)
      (bug-report-form ip))
     ;; if empty return to login page
     ((equal text "")
      ;; need function otherwise should put after macro definition
      (add-index-form ip))
     ;; otherwise save error into log file hdsri-error-log.txt
     (t
      ;; we do not use the answer (inform message)
      (omas::get-answer agent text :to-agent :CSCWD-WEB
                        :type :inform :action :log-error)
      (add-index-form ip)
      ))
    ))

;;;==============================================================================
;;;                              INDEX - HOME PAGE
;;;==============================================================================
;;; page to direct to other pages

(defun add-index-form (ip)
  (declare (special *language*))
  (format t "~%;--- posting index form... ip=~s" ip)
  (format t "~%;--- user entries: ~s" (omas::users *omas*))
  (let (;(agent-ref (omas.web::wu-get ip "agent-ref"))
        ;; the scope of *language* is restricted to the add-index-form function
        *language* language)
    ;; read language parameter
    (setq language (omas.web::wu-get ip "language"))
    ;; set global language to the parameter or to *language* as a safety feature
    (omas::set-language language)
    ;; *language* value is transferred to webdialog page or to edit forms
    (format t "~%;--- language: ~s" *language*)
    (eval
     `(html 
       (:html
        (:head
         (:title (:princ ,(select-name *ir-title*)))
         ((:meta :charset "utf-8")))
        (:body 
         "<fieldset style=\"background-color:yellow;\">"
         "<font face=\"arial\">"
         ,(cscwd-title *ir-title*)
         ;:br
         ;(:center (:h2 ,(select-name *ir-title*)))
         (:h3 (:princ ,(select-name *ir-features*)))
         ;;;         ((:table :border "0" :cols "2" :width "100%")
         ;;;          (:tbody
         ;;;           ;;=== dialogue
         ;;;           (:tr 
         ;;;            ((:td :width "10%"))
         ;;;            ((:td :width "30%") ,(select-name *ir-dialog*))
         ;;;            (:td  ((:a :href 
         ;;;                       ,(string+ "webdialog?language=" (or language *language*)))
         ;;;                   ,(select-name *ir-goto-dialog*))))
         ;;;           ))
         ;;;         
         ;;;         :hr
         ;;;         :br
         ((:table :border "0" :cols "2" :width "100%")
          (:tbody
           ;;=== Adding a new concept or editing an existing one
		   ;; the edit page is called by the browser, activating the editentry 
		   ;; function from one of the server threads. Thus, the global value of
		   ;; *language* is not that defined in the scope of this function
           (:tr
            ((:td :width "10%"))
            ((:td :width "30%") ,(select-name *ir-concept*))
            (:td ((:a :href 
                      ,(string+ "editentry?init=t&agent-ref=CSCWD-ONTO" 
                                "&language=" *language*
                                "&class-ref=CSCWD-CONCEPT"
                                "&entry-language=ALL"
                                "&caller=index"
                                ))
                  (:princ ,(select-name *ir-add/edit-concept*)))))
           
           
           ;;=== Adding a new role/link or editing an existing one
           (:tr
            ((:td :width "10%"))
            ((:td :width "30%") ,(select-name *ir-link*))
            (:td ((:a :href 
                      ,(string+ "editentry?init=t&agent-ref=CSCWD-ONTO" 
                                "&language=" *language*
                                "&class-ref=ROLE-TYPE"
                                "&caller=index"
                                ))
                  ,(select-name *ir-add/edit-link*))))
           
           ;;=== Creating a new role from a role type
           (:tr
            ((:td :width "10%"))
            ((:td :width "30%"))
            (:td ((:a :href 
                      ,(string+ "editentry?agent-ref=CSCWD-ONTO" 
                                "&language=" *language*
                                "&class-ref=ROLE"
                                "&caller=index"
                                ))
                  ,(select-name *ir-create-role*))))
           
           (:tr (:td "-"))
           ;;=== Displaying as a graph
           (:tr
            ((:td :width "10%"))
            ((:td :width "30%") ,(select-name *ir-display*))
            (:td ((:a :href ,(string+ "html/graph-display.html?language=" *language*))
                  ,(select-name *ir-display-graph*))))
           
           ;;=== Displaying as a tree
           (:tr
            (:td)
            (:td)
            (:td ((:a :href ,(string+ "html/tree-display.html?language=" *language*))
                  ,(select-name *ir-display-tree*))))
           ))
         :br
         :hr
         :br
         ((:table :border "0" :cols "2" :width "100%")
          (:tbody
           ;;=== password
           (:tr
            ((:td :width "10%"))
            ((:td :width "30%") ,(select-name *ir-password*))
            (:td ((:a :href ,(string+ "passwordform?language=" *language*)) 
                  ,(select-name *ir-password-go*))))
           ))
         :br
         :hr
         ,(select-name *ir-beta-test*)
         ((:table :border "0" :cols "2" :width "100%")
          (:tbody
           ;;=== bug report
           (:tr
            ((:td :width "10%"))
            ((:td :width "30%") ,(select-name *ir-comments*))
            (:td ((:a :href ,(string+ "bugreportform?language=" *language*)) 
                  ,(select-name *ir-comments-go*)
                  )))
           ))
         :hr
         (:small (:i "last modified: 18/9/2014"))
         ))))))

;;;================================================================================
;;;                             CONCEPT
;;;================================================================================

;;; publish-concept-resource
;;; the function is called from the web browser to retrieve all concepts (json fmt)

(defun publish-concept-resource (agent)
  "post a form to connect to a personal assistant. will only connect to known PAs.
arguments:
   agent: CSCWD-WEB
return:
   an html string corresponding to the page to publish"
  (publish 
   :path "/concept"
   :content-type "application/json; charset=utf-8"
   :function
   #'(lambda (req ent)
       (dformat :concepts 0 "~2%;========== ENTERING concept form")
       (dformat :concepts 1 "~%;--- req: ~S " req)
       ;; go get list of concepts
       (publish-concept-resource-action agent req ent))))


(defun publish-concept-resource-action (agent req ent)
  "ask CSCWD-ONTO a list of JSON coded concepts"
  (let (response parms label language)
    (setq parms (request-query req))
    (setq label (get-field "label" parms))
    (setq language (get-field "language" parms))
    
    (dformat :concepts 0 "~%;--- parms: ~%  ~S" parms)
    (dformat :concepts 0 "~%;--- label: ~%  ~S" label)
    (dformat :concepts 0 "~%;--- language: ~%  ~S" language)

    (if language
        (setq language (intern language "KEYWORD"))
      (setq language *language*)
      )
    (format t "~%@/concept resource language = ~S" language)
    (format t "~%@/concept resource label = ~S" label)
    (cond
     ;; if label was not specified then get all concept labels
     ((not label)
      (setq response (omas::get-answer agent `((:language ,language))
                                       :to-agent :CSCWD-ONTO
                                       :action :get-all-concepts-json
                                       :timeout 5
                                       :allow-nil-answer t))
      (with-http-response (req ent)
        (with-http-body (req ent)
          (format (request-reply-stream req) response)
          )
        )
      )
     ;; label was informed. get concept and related objects
     (t
      (setq response (omas::get-answer agent `((:label ,label) (:language ,language))
                                       :to-agent :CSCWD-ONTO
                                       :action :get-concept-json
                                       :timeout 5
                                       :allow-nil-answer t))
      (with-http-response (req ent)
        (with-http-body (req ent)
          (format (request-reply-stream req) response)
          )
        )
      )
     )
    )
  )
   
     

;;;-------------------------------------------------------------- publish-index

(defun publish-index (agent)
  "post an introductory index form"
  (publish 
   :path "/index"
   :content-type "text/html"
   :function
   #'(lambda (req ent)
       ;; careful: we use here agent a global variable
       (format t "~2%;========== ~s entering index form" (omas::name agent))
       (with-http-response (req ent)
         (with-http-body (req ent)
           (let ((ip (get-pa-ip req)))
             ;; here we associate the add-index-form function with the entity
             (add-index-form ip)))))))

;;;==============================================================================
;;;                             LOGIN
;;;==============================================================================
;;; form for login into the system, the initial password is the login name
;;; Each member of the CSCWD-ONTO application has a login name, a name, a first name,
;;; and an email address. All the information is handled by the MEMBERS agent.
;;; When a member logs in, MEMBERS returns the name, first name and email that
;;; are copied into the user entry.

(defun add-login-body (user-login-name)
  `(((:table :border "0" :cols "2" :width "100%")
     (:tbody
      ,(row :header (select-name *ir-username*)
            :name "login" :value user-login-name)
      :br
      ,(row :header (select-name *ir-login-language*) :name "language"
            :value "EN")
      ))))

(defun add-login-form (&key user-login-name error message)
  (format t "~%;--- posting login form...")
  (eval
   `(html 
     (:html
      (:head 
       (:title ,(select-name *ir-title*))
       ((:meta :charset "utf-8")))
      (:body 
       "<font face=\"arial\">"
       ((:form :action "ONTOCODESIGN")
        "<fieldset style=\"background-color:yellow;\">"
        :br
        ,(cscwd-title *ir-title*)
        ;(:center (:h2 ,(select-name *ir-title*)))
        ,@(when error (list (select-name *ir-no-assistant*)))
        ,@(if message `(,(format nil message))) ; removes ~
        :br
        ,@(add-login-body user-login-name)
        :br
        ,@(unless error (list (select-name *ir-login-explain*)))
        :br :br
        (:center ((:input :type "submit" :value ,(select-name *ir-send*))))
        :br
        ,@(when error (list (select-name *ir-login-error*)))
        ))))))

;;;----------------------------------------------------------- publish-login-form
;;; the function is called from within the start-server skill, one then can request
;;; a list of names and indexes from the members agent.

(defun publish-login-form (agent)
  "post a form to connect to a personal assistant. will only connect to known PAs.
arguments:
   agent: CSCWD-WEB
return:
   an html string corresponding to the page to publish"
  (publish 
   :path "/ONTOCODESIGN"
   :content-type "text/html"
   :function
   #'(lambda (req ent)
       (format t "~2%;========== ENTERING login form")
       (format t "~%;--- req: ~S " req)
       ;; organize login
       (publish-login-form-check-login agent req ent))))

;;;----------------------------------------------- publish-login-form-check-login

(defun publish-login-form-check-login (agent req ent)
  "function attached to the login-form entity triggered whenever the page is ~
   required.
     First time it checks login and password
	 Second time it posts the page and saves some data
	 Third time it creates a PA and an entry in the user list
Arguments:
   req: the request object of the entity
   ent: the entity handling the page in the server.
Return:
   an HTML string for posting the login page the first 2 times
   an HTML string for posting the index (home) page"
  (let ((parms (request-query req))
        answer result)
    (cond
     ;;== if we are entering the form for the first time, then parms
     ;; is nil, and we must require a login
     ((null parms)
      ;; ask for login/password
      (multiple-value-bind (login password) (get-basic-authorization req)
        (format t "~%;--- (request-query req): ~S " (request-query req))
        (format t "~%;--- login: ~S, password: ~S"  login password)
        (cond
         ;;-- if we have a login and password, we ask CSCWD-MEMBERS to validat access
         ((and login password)
          ;; ask MEMBERS to check login (e.g. "barthes") and password
          (setq answer
                (omas::get-answer
                 agent 
                 `(("login" . ,login) ("password" . ,password))
                 :to-agent :CSCWD-MEMBERS
                 :action :authorized?
                 :pattern '("person" ("name")("first name")("email"))
                 :timeout 5
                 :allow-nil-answer t))
          ;; in case of error or timeout we must return nil
          (setq result
                (if (and (listp answer)(eql (car answer) :error)) nil answer))
          ;; if OK, answer should be something like
          ;; (("name" "Barthès") ("first-name" "Jean-Paul") ("email" "barthes@utc.fr"))
          (format t "~%;--- result: ~S"  result)
          
          (cond
           (result
            ;; save user-info
            ;(remember agent result :user-info)
            ;(remember agent login :login-name)
            ;; and call the login page to specify the language
            (with-http-response (req ent)
              (with-http-body (req ent)
			    ;; here we go to call index directly, setting *language* to EN
                ;(add-login-form :user-login-name login))))
                (publish-login-form-action agent req ent login result))))
           
           ;; else if login and password are not accepted, then oblige the server
           ;; to put up the name/password dialog again
           (t
            (with-http-response (req ent :response *response-unauthorized*)
              (set-basic-authorization req "ONTOCODESIGN server")
              (with-http-body (req ent))))
           )
          )
         
         ;;-- else if there is some problem with login and password, we oblige the
         ;; server to put up a name/password dialog again
         (t
          (with-http-response (req ent :response *response-unauthorized*)
            (set-basic-authorization req "ONTOCODESIGN server")
            (with-http-body (req ent))))
         )
        ) ; end null parms clause
      ) ; end multiple-value 
     
     ;;== else (second time around) parms contain login and language; create PA 
     ;; and post index (home) page
	 ;; with the setting of *language* to EN we should never come here.
;;;     (t 
;;;      (with-http-response (req ent)
;;;        (with-http-body (req ent)
;;;          (publish-login-form-action agent req ent)))
;;;      )
     )))

;;;---------------------------------------------------- publish-login-form-action
;;; We create a new PA systematically by cloning the file XXXXX-<TAG> where <TAG>
;;; specifies the language.
;;; The PA is alive for the duration of the session and has no persistent storage

(defun publish-login-form-action (agent req ent user-login user-info)
  "controls the login page. We get here once login and password have been checked.
   We create a new temporary PA.
Arguments:
   agent: CSCWD-WEB
   req: request
   ent: entity handling the page
   user-login-name: login, e.g. \"barthes\" 
   user-info: ((\"name\" <name>)(\"first name\" <first name>)(\"email\" <email>))"
  (declare (ignore ent agent)(special *language*))
  (let ((*language* :EN)
        ip pa-name pa-key)
    
    (format t "~2%;---------- CSCWD-WEB: entering publish-login-form-action")
    (format t "~%;--- user-login-name: ~s" user-login)
    (format t "~%;--- user-info:  ~s" user-info)  
    
    ;; get ip of the sender of the web page
    (setq ip (get-pa-ip req))               
    (format t "~%;--- sender ip: ~s,  ~s" ip (socket:ipaddr-to-dotted ip))
        
    ;; assistant has the same name as the user login (better than the user name 
    ;; because the login is unique)
    (setq pa-name (string+ user-login "-PA-" *language*))
    (format t "~%;--- pa-name:  ~s" pa-name)
    
    (setq pa-key (intern (moss::%string-norm pa-name) :keyword))
    (format t "~%;--- pa key:  ~s" pa-key)
    
    ;;=== here we create a new PA, using the
    (create-assistant-for-CSCWD pa-key :language *language*
                                :user-description user-info)
    (format t "~%;--- Assistant created ---")
    
    ;; create an entry into the *omas* table with name and ip
    (omas.web::wu-set-user-entry
     ip
     `("user-login" . ,user-login)
     `("user-name" . ,(car (get-field "name" user-info)))
     `("user-first-name" . ,(car (get-field "first name" user-info)))
     `("user-email" . ,(car (get-field "email" user-info)))
     `("pa-key" . ,pa-key)
     `("language" . ,(symbol-name *language*))
     `("agent-ref" . ,pa-name))
    
    (format t "~%;--- (omas::users *omas*):  ~s" (omas::users *omas*))
    
    ;; return the index page
    (add-index-form ip)
    ))

#|
This function could be interesting for localizing interfaces when dialogs are used

(defun publish-login-form-action (agent req ent)
  "controls the login page. We get here once login and password have been checked.
   We create a new temporary PA.
Arguments:
   agent: CSCWD-WEB
   req: request
   ent: entity handling the page
   user-login-name: login, e.g. \"barthes\" 
   user-info: ((\"name\" <name>)(\"first name\" <first name>)(\"email\" <email>))"
  (declare (ignore ent))
  (let* ((parms (request-query req))
         (user-login (recall agent :login-name))
         (user-info (recall agent :user-info))
         (language (get-field "language" parms))
         ip pa-name pa-key)
    
    (format t "~2%;---------- CSCWD-WEB: entering publish-login-form-action")
    (format t "~%;--- parms: ~s" parms)
    (format t "~%;--- user-login-name: ~s" user-login)
    (format t "~%;--- user-info:  ~s" user-info)  
    
    ;; get ip of the sender of the web page
    (setq ip (get-pa-ip req))               
    (format t "~%;--- sender ip: ~s,  ~s" ip (socket:ipaddr-to-dotted ip))
    
    ;; set the global language variable with the input field (uppercased)
    (set-language language)
    (format t "~%;--- *language*:  ~S" *language*)
    
    ;; assistant has the same name as the user login (better than the user name 
    ;; because the login is unique)
    (setq pa-name (string+ user-login "-PA-" *language*))
    (format t "~%;--- pa-name:  ~s" pa-name)
    
    (setq pa-key (intern (moss::%string-norm pa-name) :keyword))
    (format t "~%;--- pa key:  ~s" pa-key)
    
    ;;=== here we create a new PA
    (create-assistant-for-CSCWD pa-key :language *language*
                                :user-description user-info)
    
    (format t "~%;--- Assistant created ---")
    
    ;; create an entry into the *omas* table with name and ip
    (omas.web::wu-set-user-entry
     ip
     `("user-login" . ,user-login)
     `("user-name" . ,(car (get-field "name" user-info)))
     `("user-first-name" . ,(car (get-field "first name" user-info)))
     `("user-email" . ,(car (get-field "email" user-info)))
     `("pa-key" . ,pa-key)
     `("language" . ,language)
     `("agent-ref" . ,pa-name))
    
    (format t "~%;--- (omas::users *omas*):  ~s" (omas::users *omas*))
    
    ;; return the index page
    (add-index-form ip)
    ))
|#
;;;=============================================================================
;;;                            PASSWORD FORM
;;;=============================================================================

;;;----------------------------------------------------------- add-password-body

(defun add-password-body (&key (old-password "")(new-password "")(echo ""))
  `(((:table :border "0" :cols "2" :width "100%")
     (:tbody
      (:princ ,(row :header (select-name *ir-old-pass*) :name "old-password"
                    :type "password" :value old-password))
      :br
      (:princ ,(row :header (select-name *ir-new-pass*) :name "new-password" 
            :type "password" :value new-password))
      :br
      (:princ ,(row :header (select-name *ir-new-pass-check*) :name "echo" 
            :type "password" :value echo))
      :br
      ))))

;;;----------------------------------------------------------- add-password-form

(defun add-password-form (ip &key messages old-password new-password echo)
  (declare (ignore ip))
  (eval
   `(html 
     (:html
      (:head 
       (:title :princ ,(select-name *ir-title-pass*)))
      ((:meta :charset "utf-8")))
     (:body 
      "<font face=\"arial\">"
      ((:form :action "passwordform")
       "<fieldset style=\"background-color:yellow;\">"
       :br
       ,(cscwd-title *ir-title-pass*)
       ,@(if messages (list (format nil "~{~a~^~%~}" messages)))
       :br
       ,@(add-password-body :old-password old-password :new-password new-password
                            :echo echo)
       :br
       :br
       (:center ((:input :type "submit" :value ,(select-name *ir-send*))))
       :br 
       :br
       ((:a :href "index") (:princ ,(select-name *ir-return*)))
       )))))

;;;------------------------------------------------------- publish-password-form

(defun publish-password-form (agent)
  (publish 
   :path "/passwordform"
   :content-type "text/html"
   :function
   #'(lambda (req ent)
       ;(format t "~%;++++ publish-password-form /req: ~s~%~%" req)
       ;(format t "~%;++++ publish-password-form /(request-query req): ~s~%~%" 
       (with-http-response (req ent)
         (with-http-body (req ent)
           (publish-password-form-action agent req ent))))))

;;;------------------------------------------------ publish-password-form-action

(defun publish-password-form-action (agent req ent)
  "controls the password form"
  (declare (ignore ent)(special *language*))
  ;; note that the parameters are specified using Engish.
  (let* ((query-req (request-query req))
         (language (get-field "language" query-req))
         (*language* (omas::set-language language :only-return t))
         (old-password (get-field "old-password" query-req))
         (new-password (get-field "new-password" query-req))
         (echo (get-field "echo" query-req))
         ip answer result username)
    
    ;; first time around, post page
    (unless old-password 
      (return-from publish-password-form-action (add-password-form ip)))
    
    ;; get ip of the sender of the web page
    (setq ip (get-pa-ip req))               
    (format t "~%; publish-password-form-action /sender ip: ~s,  ~s" 
      ip (socket:ipaddr-to-dotted ip))
    
    ;; get user name
    (format t "~%; publish-password-form-action /users:~% ~s"
      (omas::users *omas*))
    (setq username (get-field "user-login" (cdr (assoc ip (omas::users *omas*)))))
    (format t "~%; publish-password-form-action /user-name: ~s" username)
    
    (unless username
      (return-from publish-password-form-action 
        (add-password-form 
         :messages `(,(select-name *ir-password-no-user*)))))
    
    ;; if fields contain values, check old index
    (setq answer
          (omas::get-answer agent `(("login" . ,username)
                                    ("password" . ,old-password))
                            :to-agent :CSCWD-members
                            :action :check-password
                            :timeout 5
                            :allow-nil-answer t))
    ;; answer is either t, or (:error nil) or nil on timeout
    (setq result (if (and (listp answer)(eql (car answer) :error)) nil answer))
    (format t "~%; publish-password-form-action /checked old-password ~s" result)
    
    ;; if nil call page again
    (unless result
      (return-from publish-password-form-action 
        (add-password-form ip :messages `(,(select-name *ir-password-wrong*))
                           :new-password new-password :echo echo)))
    
    ;; check new index and echo
    (unless (moss::equal-gen new-password echo)
      (return-from publish-password-form-action 
        (add-password-form ip
                           :messages `(,(select-name *ir-password-no-check*))
                           :old-password old-password :new-password new-password)))
    
    ;; ok update index
    (setq answer
          (omas::get-answer agent `(("login" . ,username)
                                    ("password" . ,new-password))
                            :to-agent :CSCWD-members
                            :action :set-password
                            :timeout 5
                            :allow-nil-answer t))
    (setq result (if (and (listp answer)(eql (car answer) :error)) nil answer))
    (format t "~%; publish-password-form-action /set new-password ~s" result)
    
    (unless result
      (return-from publish-password-form-action 
        (add-password-form ip :messages 
                           `(,(select-name *ir-password-not-ok*)))))
    
    ;; ok, redisplay page
    (add-password-form ip :messages `(,(select-name *ir-password-ok*)))
    ))

;;;==============================================================================
;;;                         POP UP CONCEPTS PAGE
;;;==============================================================================
;;; This page contains the list of currently available keywords
;;; It posts them in the current language
;;; the list of categories should be ordered alphabetically with <BR> separations

(defun pop-up-concepts-page (concept-list &optional var)
  (let ((ret-address (or var "target")))
    (eval
     `(html
       (:html
        (:head 
         (:title "CSCWD Concepts")
         ((:meta :charset "utf-8")))
        (:body
         "<font face=\"arial\">"  
         ,@(mapcar #'(lambda (xx) 
                       (string+ "<a href=\"javascript:window.opener.provide('"
                                xx "', '" ret-address "')\">" xx "</a><BR>"))
             concept-list)
         :br
         ))))))

;;;----------------------------------------------------- publish-pop-up-concepts

(defun publish-pop-up-concepts (agent)
  (publish 
   :path "/popupconcepts"
   :content-type "text/html"
   :function
   #'(lambda (req ent)
       (with-http-response (req ent)
         (with-http-body (req ent)
           (format t "~2%;========== ~S entering pop up concepts" (omas::name agent))  
           (publish-pop-up-concepts-action agent req ent))))))

;;;---------------------------------------------- publish-pop-up-concepts-action

(defun publish-pop-up-concepts-action (agent req ent)
  (declare (ignore ent)(special *language*))
  (let* ((parms (request-query req))
         (var (get-field "var" parms))
         (language (or (get-field "language" parms) *language*))
         answer concept-list)
    
    (format t "~%;---------- ~S: Entering publish-pop-up-concepts-action"
	    (omas::name agent))
    (format t "~%;--- answer: ~S" answer)
	
    (omas::set-language language)
        
    (setq answer 
          (omas::get-answer agent language :to-agent :CSCWD-ONTO
                            :action :get-all-concepts
                            :timeout 5 :allow-nil-answer t))
    (if* answer
       then
            (setq concept-list answer)
       else
            (setq concept-list '("Can't get the keywords"))
            )
    (pop-up-concepts-page concept-list var)))


;;;=============================================================================
;;;                            WEB DIALOG FORM
;;;=============================================================================
;;; when the server uses this entity, it must decide who is calling. to do so
;;; it uses the ip of the sending machine, retrieving the corresponding pa key
;;; from the a-list stored in the users slot of the *omas* globals. if there is
;;; no entry, then transfers message to hdsri. we assume that each different
;;; caller is on a different machine. if not (e.g. for tests on the server) the
;;; new caller, e.g. untel, has priority over hdsri (the ip will be "127.0.0.1")
;;; when verbatim in non nil, then pass all chars

(defun dialog-form (&optional user-input answer verbatim)
  (if verbatim
      ;; we want to pass every character (no end of sentence on . or ? or !
      (eval
       `(html 
         (:html
          (:head (:title ,(select-name *ir-title-dialog*))
                 ((:meta :charset "utf-8")))
          (:body
           "<fieldset style=\"background-color:yellow;\">"
           "<font face=\"arial\">"
           ,(cscwd-title *ir-title-dialog*)
           ((:form :action "webdialog")
            
            ;(:center (:h2 ,(select-name *ir-title-dialog*)))
            ,@(if answer
                  (progn
                    `((:b ,(select-name *ir-dialog-answer*))
                      (:princ " ")
                      (:princ ,user-input)
                      :br
                      :br
                      (:princ ,answer)))
                `((:b ,(select-name *ir-dialog-question*))))
            :br
            :br
            (:center
             "<textarea rows=\"3\" cols=\"100%\" name=\"user-input\" autofocus>
          </textarea>")
            :br
            "<center>"
            ((:input :type "submit" :value ,(select-name *ir-send*)))
            "</center>"
            :br :br
            (:small ,(select-name *ir-dialog-explain*))
            :br
            :br
            ((:a :href "index") ,(select-name *ir-return*))
            )))))
    
    ;; otherwise end input automatically on . or ? or !
    (eval
     `(html 
       (:html
        (:head (:title ,(select-name *ir-title-dialog*))
               ((:meta :charset "utf-8")))
        (:body
         "<script language=\"javascript\">
	   function checkKey(e) {
              var character = String.fromCharCode(e.keyCode);
              if( character == '.' || character == '!' || character == '?') {
                 document.forms[0].submit();
                 }
              }
        </script>"
         "<fieldset style=\"background-color:yellow;\">"
         "<font face=\"arial\">"
         ((:form :action "webdialog")
          ,(cscwd-title *ir-title-dialog*)
          :br
          ;(:center (:h2 ,(select-name *ir-title-dialog*)))
          ,@(if answer
                (progn
                  `((:b ,(select-name *ir-dialog-answer*))
                    (:princ " ")
                    (:princ ,user-input)
                    :br
                    :br
                    (:princ ,answer)))
              `((:b ,(select-name *ir-dialog-question*))))
          :br
          :br
          (:center
           "<textarea rows=\"3\" cols=\"100%\" name=\"user-input\" 
              onkeypress= \"return checkKey(event)\" autofocus></textarea>")
          :br
          "<center>"
          ((:input :type "submit" :value ,(select-name *ir-send*)))
          "</center>"
          :br :br
          (:small ,(select-name *ir-dialog-explain*))
          :br
          :br
          ((:a :href "index") ,(select-name *ir-return*))
          )))))
    ))

;;;---------------------------------------------------------- publish-web-dialog
;;; publish web page

(defun publish-web-dialog (agent)
  (publish 
   :path "/webdialog"
   :content-type "text/html"
   :function
   #'(lambda (req ent)
       ;(format t "~%;++++ publish-web-page /req: ~s~%~%" req)
       ;(format t "~%;++++ publish-web-page /(request-query req): ~s~%~%" 
       (with-http-response (req ent)
         (with-http-body (req ent)
           (publish-web-dialog-action agent req ent))))))

;;;--------------------------------------------------- publish-web-dialog-action

(defun publish-web-dialog-action (agent req ent)
  "functions associated to the entity handling the webdialog page"
  (declare (ignore ent))
  (let* ((parms (request-query req))
         ;; parms contains whatever was typed by the user in user-input
         (user-input (get-field "user-input" parms))
         ;; there is no "language" parameter in parms!
         ;(language (get-field "language" parms))
         (ip (socket:remote-host (request-socket req)))
         ;; ... but there might be one in the user-input
         (language (omas.web::wu-get ip "language"))
         data text pa-key edit-data verbatim)
    ;; set language
    (set-language language)
    (cond
     ((and user-input (setq pa-key (get-pa-key req)))
      ;; here we got some question from the user and its ip
      (format t "~%;++++ publish-web-dialog-action /user-input: ~s~%~%" user-input)
      
      ;; send the request as an internal message to the web agent
      ;; with the text and the identity of the pa
      ;; get-answer will create a gate to wait for any processing 
      ;; that must be done to compute the answer
      ;; default action is :process-web-text
      (setq data 
            (omas::get-web-answer :cscwd-web user-input :to-agent pa-key))
      ;; answer returns a list of pairs 
      ;;  ((:text "")(:question "")(:answer ""))
      ;; that may be inserted into the answer page==
      (format  t "~%; publish-web-dialog-action /answer: ~s" data)
      
      ;;== when the answer is a list containing :data, then we launch an editor
      (cond
       ((setq edit-data (cdr (assoc :data data)))
        ;; call editor, return data is like (:create "stevens-web" "news item")
        ;;*****
        ;; calling the edit post will post a page, however, it is not clear as to
        ;; what entity will handle the form when clicking the "send" button
        ;;*****
        ;; we skip te first returnd value...
        ;;;        (net.aserve.client:do-http-request "http://localhost/locate"
        ;;;          :query `(("agent" . ,(cadr edit-data))
        ;;;                   ("class-ref" . ,(caddr edit-data))))
        ;;;********* not sure the agent is the right one
        (omas.web::web-edit-external-entry agent ip
                                           `(("agent-ref" . ,(cadr edit-data))
                                             ("class-ref" . ,(caddr edit-data))))
        )
       ;;== otherwise, recover text containing answer and question
       (t
        (setq text (cdr (assoc :text data)))
        ;; check if we want to pass all chars
        (setq verbatim (getvl data :verbatim))
        ;; print the page with the result underneath
        (dialog-form user-input text verbatim))
       )
      )
     
     ;;=== here we have a user input but could not locate pa
     (user-input
      (dialog-form user-input (select-name omas::*web-no-pa*))
      )
     
     ;;=== we come here the first time around since 
     (t
      ;; first time access, put up the form
      (dialog-form)))))



;;;===============================================================================
;;; 
;;;                               INITIAL CONDITIONS 
;;;
;;;===============================================================================

;;; none

;;;==============================================================================
;;; 
;;;                                    GOALS
;;;
;;;==============================================================================

;;; goal to start the server a few seconds after the agent has been created

(defun start-server-script (agent)
  "send a request message to start the web server"
  (declare (ignore agent))
  (list (make-instance 'omas::message :type :request :to :cscwd-web 
          :action :start-server)))

(defgoal :start-server :cscwd-web
  :activation-delay 2 
  :script start-server-script)

;;; ================================== setup messages ===========================

;;; none

:eof
