﻿;;;-*- Mode: Lisp; Package: "CIT-ADDRESS" -*-
;;;===============================================================================
;;;11/04/04
;;;                               AGENT CIT-ADDRESS
;;;
;;;===============================================================================

;;; we call our agent CIT-ADDRESS to avoid conflicts sith the ADDRESS symbol used in
;;; the ontology

#|
2007
 0831 adding a few functions to select using all words in the input
 0901 shift most functions into MOSS services
 1017 adding :what-is skill
2008
 0605 checking failures on unknown persons modifying get-phone-fill-pattern
2009
 0209 adding master option to agent definition
2010
 0529 upgrading  to OMAS v8, goal section has also been modified
2011
 0404 clean up
|#

(defpackage :CIT-ADDRESS (:use :cl #+MCL :ccl :moss :omas))
(in-package :CIT-ADDRESS)

(omas::defagent :CIT-ADDRESS)

;;;===============================================================================
;;; 
;;;                         service macros and functions 
;;;
;;;===============================================================================


;;; ==================================== globals =================================
;;; Globals  can be used within the agent space to simplify programming, although
;;; it is better to use the agent memory.

(defParameter *address-words* 
  '("address" "home address" "private address" "live" "lives"))

(defParameter *empty-words* 
  '("the" "a" "an" "of" "from" "to" "those" "these" "that" "s" "?"))

(defParameter *phone-words*
  '("phone" "telephone" "phone-number" "telephone-number" "number"))

(defParameter *default-person-phone-pattern*
  '("person" ("name")("first name")
    ("telephone" ("telephone"))))

(defParameter *default-organization-phone-pattern*
  '("organisation" ("name") ("acronym")
    ("telephone" ("telephone"))))

;;;===============================================================================
;;; 
;;;                                    skills 
;;;
;;;===============================================================================

;;;========================================================================== SKILL
;;;                                   GET-ADDRESS
;;;================================================================================

(defskill :GET-address :CIT-ADDRESS
  :static-fcn static-GET-ADDRESS
  )

;;;------------------------------------------------------------ STATIC-GET-ADDRESS

;;; the strategy is to look for people, organizations or meetings, namely objects 
;;; that can have an address accessing them from entry points.

(defun static-get-address (agent message args)
  "We receive a request for an address. We try to obtain the info from the ~
  data. If no pattern, returns a list of strings, otherwise a list of ~
  filled patterns.
Arguments:
   agent: :CIT-ADDRESS
   message: unused
   args: e.g. ((:data \"of\" \"Barthes\")(:pattern ...))
Return:
   :failure or a list of strings or a list of filled patterns."
  (declare (ignore message))
  ;; first we should get some entry-point from data
  ;; remove empty words from the data
  (let ((input (cdr (assoc :data args)))
        ;; set the local language temporarily
        (*language* (or (cdr (assoc :language args)) *language*))
        (pattern (cdr (assoc :pattern args)))
        data answer results)
    ;(format *debug-io* 
    ;    "~&+++ :CIT-ADDRESS static-get-address /data: ~S in package: ~S~:
    ;     ~&pattern: ~S" data *package* pattern)
	 (setq data (moss::%clean-word-list input *address-words*))
	 (format t "~%; static-get-address /data:~% ~S" data)
	;;===== when no pattern try person, organization and meetings
	(unless pattern
	  ;;=== first try to get persons from the input
	  (setq results (locate-objects data "person"))
	
	  ;;=== if no results try organizations
	  (unless results
	    (setq results (locate-objects data "organization")))
		
	  ;;=== if no result try meetings
	  (unless results
	    (setq results (locate-objects data "meeting")))

	  ;;=== if no result, we give up
	  ;; when there is a result get answer string
	  (unless results
	    (return-from static-get-address (static-exit agent :failure)))
	   
	  ;; otherwise we get address strings
	  (setq answer (broadcast results '=get-address))
	  (return-from static-get-address (static-exit agent answer))
	  )
		
	;;===== When there is a pattern, the first element specifies the class
    (setq results (locate-objects data (car pattern)))
	
	;;=== if no result give up
	(unless results
	    (return-from static-get-address (static-exit agent :failure)))
		
	;; otherwise fill pattern and return
	(setq answer (fill-pattern results pattern))
	(static-exit agent answer)))

#|
(static-get-address SA_CIT-ADDRESS nil 
    '((:data "address" "of" "barthès" "?")(:language . :EN)))
|#
;;;========================================================================== SKILL
;;;                               GET-HOME-ADDRESS
;;;================================================================================

(defskill :GET-home-address :CIT-ADDRESS
  :static-fcn static-GET-HOME-ADDRESS
  )

(defun static-get-home-address (agent message args)
  "We receive a request for a home address. We try to obtain the info from the ~
  data. If no pattern, returns a list of strings, otherwise a list of ~
  filled patterns.
Arguments:
   agent: :CIT-ADDRESS
   message: unused
   args: e.g. ((:data \"of\" \"Barthes\")(:pattern ...))
Return:
   :failure or a list of strings or a list of filled patterns."
  (declare (ignore message))
  ;; first we should get some entry-point from data
  ;; remove empty words from the data
  (let ((input (cdr (assoc :data args)))
        ;; set the local language temporarily
        (*language* (or (cdr (assoc :language args)) *language*))
        (pattern (cdr (assoc :pattern args)))
        data answer results)
    ;(format *debug-io* 
    ;    "~&+++ :CIT-ADDRESS static-get-address /data: ~S in package: ~S~:
    ;     ~&pattern: ~S" data *package* pattern)
	 (setq data (moss::%clean-word-list input *address-words*))
	 (format t "~%; static-get-home-address /data:~% ~S" data)
	 
	 ;;=== first case with no pattern
	 (unless pattern
	   (setq person-list (locate-objects data "person"))
	 
	   (when person-list
	    (setq answer (broadcast person-list '=get-home-address))
	    (return-from static-get-home-address (static-exit agent answer))
	    )
		;; otherwise failure
	    (return-from static-get-home-address (static-exit agent :failure))
		)
		
		
	;;===== When there is a pattern, the first element specifies the class
	;; check that car of pattern is a valid class
	(unless (moss::%%get-id (car pattern) :concept)
	    (return-from static-get-home-address (static-exit agent :failure)))
	   
    (setq results (locate-objects data (car pattern)))
	
	;;=== if no result give up
	(unless results
	    (return-from static-get-home-address (static-exit agent :failure)))
		
	;; otherwise fill pattern and return
	(setq answer (fill-pattern results pattern))
	(static-exit agent answer)))
	
;;;========================================================================== SKILL
;;;                                GET PHONE NUMBER
;;;================================================================================

;;; Strategy to get a phone number.
;;; We get a list of words or sentence that can be:
;;; 1. - numéro de téléphone de barthès?
;;; 2. - phone Dominique?
;;;or, in French
;;; 3. - téléphone du président de l'UTC
;;; 4. - téléphone du CNRS?
;;;
;;; In général the mention of phone is the goal of the request and starts the 
;;; sentence.
;;; If we plug the sentence directly into MOSS, then we'll obtain:
;;; 1. ($E-PHONE ($S-PERSON-PHONE.OF 
;;;                    (> 0) ($E-PERSON ($T-PERSON-NAME :IS "Barthès"))))
;;; 2. ($E-PHONE
;;;     ($S-PERSON-PHONE.OF (> 0) ($E-PERSON ($T-PERSON-FIRST-NAME :IS "Dominique"))))
;;; 3. ($E-PHONE
;;;     ($S-PERSON-PHONE.OF (> 0)
;;;      ($E-PERSON
;;;       ($S-TEACHING-ORGANIZATION-PRESIDENT.OF 
;;;                          (> 0) ($E-TEACHING-ORGANIZATION #)))))
;;; 4. ($E-PHONE
;;;     ($S-ORGANIZATION-PHONE.OF (> 0)
;;;      ($E-ORGANIZATION ($T-ORGANIZATION-ACRONYM :IS "UTC"))))
;;;
;;; Now if we access the KB  using this query we get:
;;; 1. ($E-OFFICE-PHONE.1 $E-CELL-PHONE.1 $E-HOME-PHONE.1 $E-OFFICE-PHONE.2 
;;;     $E-CELL-PHONE.2 $E-HOME-PHONE.2 $E-CELL-PHONE.3)
;;;   i.e. all the telephone number of all the barthès
;;; 2. ($E-HOME-PHONE.1 $E-OFFICE-PHONE.2 $E-CELL-PHONE.2)
;;; 3. ($E-CELL-PHONE.5)
;;; 4. ($E-OFFICE-PHONE.4)
;;;
;;; We need to provide more information when there is more than one answer, first
;;; distinguishing among people and organizations.

(defskill :GET-PHONE-NUMBER :CIT-ADDRESS
  :static-fcn static-GET-PHONE-NUMBER
  )

(defun static-get-phone-number (agent message args)
  "We receive a request for a phone number. We try to obtain the info from the ~
   data.
Arguments:
   agent: :CIT-ADDRESS
   message: unused
   args: alist e.g. ((:data \"of\" \"Barthes\") (:pattern ...))
Return:
   :failure or filled pattern."
  (declare (ignore message))
  ;; first we should get some entry-point from data
  (let ((data (cdr (assoc :data args)))
        (pattern (cdr (assoc :pattern args)))
        (*language* (or  (cdr (assoc :language args)) *language*))
        query-list results entry-list)
    ;(format *debug-io* 
    ;    "~&+++ :CIT-ADDRESS static-get-phone-number /data: ~S package: ~S~
    ;     ~&pattern: ~S language ~S" data *package* pattern *language*)
    ;; access entities by trying to build a formal query
    (setq query-list (moss::path-build-query-list data))
    ;(format *debug-io* "~&+++ CIT-ADDRESS static-get-phone-number/ query-list:~% ~S" 
    ;  query-list)
    ;; try all answers
    (dolist (query query-list)
      ;; we want to avoid returning ALL telephone numbers when the owner has not been
      ;; identified
      (when (cdr query)
        ;; then using the query try to access phones...
        (setq entry-list (access query :already-parsed t))
        ;(format *debug-io* "~&+++ CIT-ADDRESS static-get-phone-number/ results:~% ~S" 
        ;  results)
        (setq results (append results entry-list))))
    ;(format *debug-io* "~&+++ CIT-ADDRESS static-get-phone-number/ collected results:~% ~S" 
    ;  results)
    ;; if we have results then we won, put the result into a nice format
    (setq results
          (if results (get-phone-fill-pattern (delete-duplicates results) pattern)))
    ;(format *debug-io* "~&+++ CIT-ADDRESS static-get-phone-number/ final results:~% ~S" 
    ;  results)
    ;; return the answer
    (static-exit agent (or results "failure"))))

;;;------------------------------------------------------ GET-PHONE-FILL-PATTERN
;;; From the phone numbers must produce something like:
;;;  ("téléphone du domicile" "0344303137"
;;;     ("personne" ("nom" "Barthes") ("prénom" "Jean-Paul"))
;;;     ("personne" ("nom" "Barthes" "Biesel") ("prénom" "Dominique")))
;;; so that we can print:
;;;  0344303137 (domicile: Jean-Paul Barthès, Dominique Barthès Biesel)

(defun get-phone-fill-pattern (object-list &optional pattern)
  "accesses the local database using the computed access list ~
   e.g. ($E-HOME-PHONE.1 $E-OFFICE-PHONE.2).
   Returns only info for any object having a phone property.
Arguments:
   access-list: list of objects representing telephone numbers
   pattern (opt): pattern to fill with data from objects
Return:
   a tree structure of strings reflecting the MOSS structure."
  (declare (ignore pattern))
  (let (owners results entry description)
    ;; debug: check object-list
    ;(format *debug-io* "~&+++ get-phone-fill-pattern /object-list: ~&  ~S"
    ;        object-list)
    ;; for each phone, get the owner
    (dolist (phone object-list)
      ;; first get the data concerning the number
      (setq entry 
            (moss::fill-answer-pattern-for-one-level 
             (case *language*
               (:fr '("téléphone"))
               (:en '("phone")))
             phone))
      ;(print `(+++entry ,entry))
      (when entry
        ;; then get owners
        (setq owners (send phone '=get 'IS-PHONE-OF))
        ;; for each owner get the relevant info
        (dolist (owner owners)
          (setq description
                (append
                 ;; get type
                 (send (car (moss::%type-of owner)) '=summary)
                 ;; get summary
                 (list (send owner '=summary))))
          ;; update entry
          (setq entry (append entry (list description))))
        (push entry results)))
    (reverse results)))

#|
(get-phone-fill-pattern '($E-HOME-PHONE.1))
(("téléphone du domicile" ("+33 (0)3 44 23 31 37")
  ("personne" ("Barthès: Jean-Paul"))
  ("personne" ("Barthès, Barthès-Biesel: Dominique"))))

(get-phone-fill-pattern '($E-OFFICE-PHONE.1 $E-CELL-PHONE.1 $E-HOME-PHONE.1 $E-OFFICE-PHONE.2 
     $E-CELL-PHONE.2 $E-HOME-PHONE.2 $E-CELL-PHONE.3))
(("téléphone professionnel" ("+33 (0)3 44 23 44 66")
  ("personne" ("Barthès: Jean-Paul")))
 ("portable" ("+33 (0)6 80 45 32 67") ("personne" ("Barthès: Jean-Paul")))
 ("téléphone du domicile" ("+33 (0)3 44 23 31 37")
  ("personne" ("Barthès: Jean-Paul"))
  ("personne" ("Barthès, Barthès-Biesel: Dominique")))
 ("téléphone professionnel" ("+33 (0)3 44 23 43 93")
  ("personne" ("Barthès, Barthès-Biesel: Dominique")))
 ("portable" ("+33 (0)6 81 24 46 66")
  ("personne" ("Barthès, Barthès-Biesel: Dominique")))
 ("téléphone du domicile" ("+86 571 8701 0782") ("personne" ("Barthès: Camille")))
 ("portable" ("+86 134 5678 3173") ("personne" ("Barthès: Camille"))))

? (get-phone-fill-pattern '($E-OFFICE-PHONE.4))
(("téléphone professionnel" ("+33 3 44 32 44 23") ("université" ("UTC "))))
|#
;;;========================================================================== SKILL
;;;                                   WHAT-IS
;;;================================================================================

(defskill :WHAT-IS :CIT-ADDRESS
  :static-fcn static-WHAT-IS
  )

(defun static-what-is (agent message args)
  "We receive a request for info on a person.
Arguments:
   agent: :CIT-ADDRESS
   message: message
   args: e.g. ((:data \"of\" \"Barthes\"))
Return:
   :failure or e.g. (\"person\" (\"name\" \"barthes\"))."
  (declare (ignore message))
  ;; first we should get some entry-point from data
  ;; remove empty words from the data
  (let ((data (cdr (assoc :data args)))
        (*language* (or (cdr (assoc :language args)) *language*))
        query-list object-list results)
    ;(format *debug-io* "~&+++ CIT-ADDRESS WHAT-IS /data: ~S in package: ~S"
    ;  data *package*)
    (unless data
      (return-from static-WHAT-IS (static-exit agent "failure")))
    ;; try to obtain a formal query from the ontology and the KB
    (setq query-list (moss::path-build-query-list data))
    ;(format *debug-io* "~&+++ CIT-ADDRESS WHAT-IS /query-list: ~S" query-list)
    ;; if nothing, do not need to answer
    (unless query-list
      (return-from static-WHAT-IS :abort))
    ;;  otherwise apply all queries merging results
    (dolist (query query-list)
      (setq object-list (append object-list (access query :already-parsed t))))
    ;(format *debug-io* "~&+++ CIT-ADDRESS WHAT-IS /object-list: ~S" object-list)
    ;; if we got something here, we should access the publications, counts them,
    (setq results 
          (broadcast 
           (delete-duplicates object-list)
           '=get-definition-summary))
    ;(format *debug-io* "~&+++ CIT-ADDRESS WHAT-IS /results: ~S" results)
    (static-exit agent (or results "failure"))))

;;;========================================================================== SKILL
;;;                                   WHO-IS
;;;================================================================================

(defskill :WHO-IS :CIT-ADDRESS
  :static-fcn static-WHO-IS
  )

(defun static-who-is (agent message args)
  "We receive a request for info on a person.
Arguments:
   agent: :CIT-ADDRESS
   message: message
   args: e.g. ((:data \"of\" \"Barthes\"))
Return:
   :failure or e.g. (:address \"03 44 20 31 37\")."
  (declare (ignore message))
  ;; first we should get some entry-point from data
  ;; remove empty words from the data
  (let ((data (cdr (assoc :data args)))
        (*language* (or (cdr (assoc :language args)) *language*))
        query-list person-list results)
    (moss::vformat "~&+++ CIT-ADDRESS WHO-IS /data: ~S in package: ~S"
      data *package*)
    (unless data
      (return-from static-who-is (static-exit agent :failure)))
    ;; try to obtain a formal query from the ontology and the KB
    (setq query-list (moss::path-build-query-list data))
    (moss::vformat "~&+++ CIT-ADDRESS WHO-IS /query-list: ~S" query-list)
    ;; if nothing, do not need to answer
    (when (or (null query-list) (equal query-list '(nil)))
      (return-from static-who-is (static-exit agent :failure)))
    ;;  otherwise apply all queries merging results
    (dolist (query query-list)
      (setq person-list (append person-list (access query :already-parsed t))))
    (moss::vformat "~&+++ CIT-ADDRESS WHO-IS /person-list: ~S" person-list)
    ;; if we got something here, we should access the publications, counts them,
    (setq results 
          (broadcast 
           (delete-duplicates person-list)
           '=get-documentation :no-summary t))
    (moss::vformat "~&+++ CIT-ADDRESS WHO-IS /results: ~S" results)
    (static-exit agent (or results :failure))))

#|
;;; ================================= skill section ==============================

(defParameter *CIT-ADDRESS-xxx-functions*
  '((:functions static-XXX 
                dynamic-XXX)))

(defskill :XXX CIT-ADDRESS
  :static-fcn static-XXX
  :dynamic-fcn dynamic-XXX
  :time-limit-fcn time-limit-XXX
  :timeout-handler timeout-handler-XXX
  :preconditions preconditions-XXX
  :select-best-answer-fcn select-best-answer-XXX
  :acknowledge-fcn acknowledge-XXX
  )

(defun static-xxx (agent message <args>)
  "documentation"
  (static-exit agent <result>))

(defun dynamic-xxx (agent message <args>)
  "documentation"
  (dynamic-exit agent <result>))

(defun time-limit-xxx (agent message message)
  "documentation"
  )

(defun timeout-handler-xxx (agent message)
  "documentation"
  )

(defun preconditions-xxx (agent message <args>)
  "documentation"
  )

(defun select-best-answer-xxx (agent message answer-message-list)
  "documentation"
  )

(defun acknnowledge-xxx (agent message<args>)
  "documentation"
  )
|#

;;;===============================================================================
;;; 
;;;                               initial conditions 
;;;
;;;===============================================================================

:ENV

;;;==============================================================================
;;; 
;;;                                    goals
;;;
;;;==============================================================================

;;; ================================= goal section ==============================

#|
"macro that builds a goal structure for an agent.
Arguments:
   goal-name:              name of the goal that will be used to create task-ids
   agent-name:             name of the agent to receive the goal
keys arguments:
   mode:                   activation mode (:rigid or :flexible)
   type:                   type of goal (:permanent :cyclic :1-shot)
   period:                 period for cyclic goals (default is 20s)
   expiration-date:        date at which the goal dies
   expiration-delay:       time to wait until we kill the goal
   importance:             on a 1-100 scale (not yet available)
   urgency:                on a 1-100 scale (not yet available)
   activation-date:        date at which the goal should fire (default is now)
   activation-delay:       time to wait before activating the goal
   activation-level:       on a 1-100 scale (default 50)  (not yet available))
   activation-threshold:   on a 1-100 scale (default 50)  (not yet available)
   activation-change-fcn:  optional function called at each cycle (not yet available)
   status:                 waiting, active, dead,... 
                              may not be useful with activation
   goal-enable-fcn:        function checking the goal enabling conditions
   script:                 script describing the goal, currently a function that
                           returns a list of messages
Return:
   goal-name."

;;; return T to enable the goal NIL ti inhibit it

(defun enable-ggg (agent script)
  "documentation"
  t)

(defun goal-ggg-CIT-ADDRESS ()
  (list (make-instance 'omas-message :type :request
                       :from :CIT-ADDRESS :to :CIT-ADDRESS
                       :action
                       :args
                       :task-id :GGG
                       )))

;;; as soon as defgoal is executed goal is launched

(defgoal GGG :CIT-ADDRESS
  :mode          
  :type 
  :period
  :expiration-date
  :expiration-delay 
  :activation-date 
  :activation-delay 
  :status 
  :goal-enable-fcn 
  :script goal-ggg-CIT-ADDRESS
  )
|#

:EOF