;;;-*- Mode: Lisp; Package: "CG-USER" -*-
;;;===============================================================================
;;;12/02/28
;;;           U T C <-> O M A R - T E S T S (utc2cit-tcp-tests.lisp)
;;;
;;; Low level tests for checking remote TCP connections
;;;===============================================================================

#|
2012
 0228 creation
|#

(in-package :cg-user)

(require :sock)

(defparameter *site* :UTC)
(defparameter *site-ip* "195.83.154.22" "UTC external ip")

(defparameter *site-port* 52008)

(defparameter *remote* :CIT)
(defparameter *remote-ip* "187.139.67.115" "OMAR ip")

;;;===============================================================================
;;;
;;;         Opening socket for letting other people connect to our site
;;;
;;;===============================================================================

(defparameter *receive-socket* nil)

(defun open-socket-for-connection ()
  (setq *receive-socket*
        (socket:make-socket :connect :passive :local-port *site-port* 
                            :backlog 50))
  (if *receive-socket*
     (format t "~%... receive-socket created: ~S" *receive-socket*)
	 (format t "~%... could not create receive-socket"))
  *receive-socket*)

;;;===============================================================================
;;;
;;;         Creating process to receive external messages
;;;
;;;===============================================================================

(defparameter *receiving-process* nil)
(defparameter *in-stream* nil)

(defun create-receiving-process ()

  (open-socket-for-connection)
  
  (setq *receiving-process*
        (mp:process-run-function 
         (format nil "~S Receiving" *site*) #'receiving *site-port*))
  (if *receiving-process*
      (format t "~%... receiving process created waiting on port: ~S" 
	          *site-port*)
	  (format t "~%... could not create receiving process."))
  *receiving-process*)

(defun receiving (port &aux message-string)
  "function called by the process"
  (loop
    ;; print trace into Lisp console
    (format t "~&=====~&~S/ receiving/ ready for incoming message..." *site*)
      
    ;; create new stream for each connection request 
    (setq *in-stream* (socket:accept-connection *receive-socket*))
    (if *in-stream*
	    (format t "~&... in-stream created: ~S" *in-stream*)
		(format t "~%... could not create in-stream."))
    
    ;; get incoming message, returning nil if message is empty (:eof reached)
    (setq message-string (read *in-stream* nil nil))
    ;; close the stream after each received message
    (close *in-stream*)
    (format t "~&... closing in-stream")
    
    ;; print trace into Lisp console
    (format t "~&=====~&~S/ receiving/ incoming message: ~&     ~S"
      *site* message-string)
    ))

;;;===============================================================================
;;;
;;;                 Sending messages to remote site
;;;
;;;===============================================================================

(defvar *send-socket* nil)

(defparameter *count* 0)

(defun create-send-socket ()
  (setq *send-socket*
        (socket:make-socket :remote-host *remote-ip* :remote-port 52008))
  (if *send-socket*
      (format t "~%... send-socket created: ~S" *send-socket*)
	  (format t "~%... could not create send-socket."))

(defun close-send-socket ()
  (close *send-socket*)
  (setq *send-socket* nil)
  :done)

(defun send-new-message ()
  
  ;; if socket is opened close it
  (when *send-socket* 
        (close *send-socket*)
		(setq *send-socket* nil)
		(format t "~%... send socket closed."))
  
  ;;; create socket-id
  (setq *send-socket* 
        (socket:make-socket :remote-host *remote-ip* :remote-port 52008))
  (if *send-socket*
      (format t "~%... send-socket created: ~S" *send-socket*)
	  (format t "~%... could not create send-socket."))
  
  ;; if no error send message, writing message to stream
  (format *send-socket* "~S" 
    (format nil "Test message from ~S #~S." *site* (incf *count*)))
  (format t "~%... writing message ~S to socket..." *count*)
	
  ;; make sure the buffer is emptied
  (force-output *send-socket*)
  
  ;; close the socket before leaving
  (close *send-socket*)
  (format t "~%... send socket now closed.")
  
  (setq *send-socket* nil)
  )

;;;===============================================================================
;;;
;;;         Killing process and closing socket for reset
;;;
;;;===============================================================================

(defun reset-everything ()
  (when *in-stream* (close *in-stream*)
    (setq *in-stream* nil))
  (when *receive-socket* (close *receive-socket*)
    (setq *receive-socket* nil))
  (when *receiving-process* (mp:process-kill *receiving-process*)
    (setq *receiving-process* nil))
  (when *send-socket* (close *send-socket*)
    (setq *send-socket* nil))
  )

;;;===============================================================================
;;;
;;;         Tests
;;;
;;;===============================================================================

#|
;;;========== STEP 1: INITIALIZE LOCAL SERVER

;;; create a socket for receiving messages
;;; create process and allow clients to connect on the socket

(create-receiving-process)

;;; this prints a message in the console:
;;;
;;; =====
;;; :UTC/ receiving/ ready for incoming message...
;;; 

;;;========== STEP 2: SENDING LOOP

;;; Wait until the other end has created a receiving process (server)
;;; ... then send a message (each time the message number increases)

(send-new-message)

;;; If everyting is OK, on can send as many messages as one wants
;;; In case of error it prints an error message. See below.


;;;========== STEP 3: RECEIVING

;;; The receiving function whenever a message is received prints:;;; The receiving function on the other site should print into the Lisp console:
;;;
;;; "=====
;;;  UTC/ receiving/ incoming message: 
;;;       Test message from <remote-site> #nn.


;;;========== STEP 4: RESET

;;; If something goes wrong the following function resets everything

(reset-everything)

;;; one can then start all over again at Step 1.




;;;=========== IN CASE OF ERROR

;;; try opening a socket on remote site prior to sending a message
;;; Remote site should have a socket waiting for a connection on the socket
(create-send-socket)

;;; if it does not work, it means that the remote site is not available or is
;;; protected by a firewall, or there is an error in the IP or in the port number
;;; Try to do a ping for checking the availabiity of the remote machine

;;; If it works, the socket is recorded in the following variable:
*send-socket* 

;;; CLOSE IT before trying to send a message
(close-send-socket)

|#
:EOF