﻿;;;-*- Mode: Lisp; Package: "ARISTIDE" -*-
;;;===============================================================================
;;;10/11/22
;;;             AGENT ARISTIDE: TASKS (file ARISTIDE-tasks.lisp)
;;;             Copyright Barthes@UTC, 2009
;;;
;;;=============================================================================== 

#|
ARISTIDE is the French assistant agent of MR. Aristide speaks only French.
It can find email addressses, telephone numbers, or use web services (?)

Its ontology is used to process dialogs, answer requests or trigger actions, i.e.
send messages to other agents.
The current file is its task library. It contains a model of all tasks the agent
can execute, related to questions asked to ARISTIDE:
   - what is the telephone number of "Trigano"?
   - phone "dbb"?
   - home phone of "Moulin"?
   - extension of DICIT?
   - send message to "gapenne".
   - ...

A task can be coupled with a dialog used to fill the task pattern (T-CONTEXT). The 
mechanism is similar to the ACTION mechanism of the MOSS dialogs.

2009
 0730 creation by cloning Albert
2010
 1122 upgrading to v8.0.8
|#

(in-package :ARISTIDE)

;;;=============================================================================== 
;;;   
;;;                               TASK MODEL
;;;
;;;===============================================================================

#|
(defconcept (:en "task argument")
  (:doc :en "A TASK ARGUMENT is an argument (required or optional) needed to complete ~
             the task. Questions are attached to the arguments so that they can be ~
             asked from the master if needed.")
  (:att (:en "arg value"))
  (:att (:en "arg name") (:unique))
  (:att (:en "arg type"))
  (:att (:en "arg key") (:unique))  ; used in the messages as a prefix
  (:att (:en "arg question"))
  )
|#

(defconcept (:en "task index")
  (:doc :en "A TASK INDEX is a property of a task. The index attribute specifies ~
             the term being used, the weight attribute gives its importance for the ~
             task.")
  (:att (:en "index"))
  (:att (:en "weight"))
  )

(definstmethod =summary TASK-INDEX ()
  (list (has-index)(has-weight)))

(defconcept (:en "task pattern")
  (:att (:en "pattern"))
  (:att (:en "weight"))
  )

(defconcept (:en "task")
  (:is-a moss::$QTSKE) ; used for linking inheritage ?
  (:doc :en "A TASK is a model of task to be performed by the assistant.")
  (:att (:en "task name") (:unique)(:entry))
  (:att (:en "dialog"))
  (:att (:en "where to ask")) ; e.g. :ADDRESS
  (:att (:en "fail question"))
  (:rel (:en "index pattern")(:to "task index"))
  (:att (:en "performative") (:id $PRFT)(:one-of :request :assert :command :answer))
  (:att (:en "message action")) ; e.g. :get-address
  )

(definstmethod =summary TASK ()
  (HAS-TASK-NAME))

;;;=============================================================================== 
;;;   
;;;                               TASK LIBRARY
;;;
;;;=============================================================================== 

;;; we could use weights for indexes...

;;;=========================================================================== ASK

(deftask "ask agent"
  :doc  "Tâche (requête) pour demander une information à un autre agent"
  :performative :command
  :dialog _ask-conversation
  :indexes ("demander à" .8 "demander" .7 "demande" .7 "demandez" .7
            "message pour" .7 "message" .4)
  )

;;;======================================================================= EXPLAIN 

(deftask "define concept"
  :doc  "Tâche (ordre) donnant la définition d'un concept de l'ontologie."
  :performative :command
  :dialog _explain-conversation
  :indexes ("définir" .6)
  )

;;;=============================================================== GETTING ADDRESS 
#|
(defindividual "task argument"
  ("arg name" "get address data")
  (:doc :en "The GET-ADDRESS-DATA argument contains a list of words to designate ~
             the owner whose address we want.")
  ("arg question" "What is the name again?")
  ("arg key" :data)
  (:var _get-address-data)
  )
|#

(deftask "get address"
  :doc "Tâche permettant d'obtenir les adresses de quelqu'un"
  :dialog _get-address-conversation
  ;("optional args" _get-address-data)
  :indexes ("adresse" .5 "adresse de" .7 "se trouve" .6 "se passe" .6 "a lieu" .6
            "ont lieu" .6 "se passent" .6 "où" .2)
  ;:where-to-ask :address
  ;("fail question" "L'adresse de qui ?")
  )

;;;===================================================== GETTING BIBLIO REFERENCES

(deftask "get biblio references"
  :doc "Tâche (ordre) permettant d'obtenir des références biblio.."
  :dialog _get-biblio-conversation
  :indexes ("papier" .5 "papiers" .5 "article" .6 "articles" .6 "proceedings" .7
            "publications" .8 "publié" .6 "publiés" .6 "mémos" .6
            "memos" .6 "manuels" .6 "rapports internes" .6 "biblio" .6
            "références" .4 "conférences" .3)
  )

;;;========================================================== GETTING HOME ADDRESS 
;;; must be updated for MCL

(deftask "get home address"
  :doc "Tâche permettant d'obtenir l'adresse personnelle de quelqu'un"
  :dialog _get-home-address-conversation
  :indexes ("addresse privée" .7 "domicile" .4 "habite" .7 "vit" .7
            "adresse personnelle" .7 "où" .2)
  )

;;;================================================ GETTING DEFINITION INFORMATION

;;;======================================================= GETTING NEWS BY SECTION

(deftask "get news by section"
  :doc "Tâche permettant obtenir nouvelles d'une page spécifique"
  :dialog _get-news-by-section-conversation
  :indexes ("nouvelles" .5 "nouvelle" .5 "dernières" .4 "dernière" .25
            "fait" .2 "faits" .2 "événement" .2 "événements" .2 "neuf" .3
            "neuve" .25 "neuves" .25 "de neuf" .5 "page" .5 "page" .5        
            "section" .5 "en relation" .5 "reliés" .5 "reliées" .5 "sur" .3)
  ;("where to ask" :news)
  ;("fail question" "Quelle est le sujet?")
  )

;;;========================================================= GETTING NEWS SECTIONS

(deftask "get news sections (categories)"
  :doc "Tâche permettant obtenir la liste des catégories de nouvelles"
  :dialog _get-news-sections-conversation
  :indexes ("nouvelles" .3 "nouvelle" .3  "section" .4 "sujet" .4 "sujets" .4
            "catégories" .4 "types" .4 "rubriques" .7)
  )

;;;================================================= GETTING TELEPHONE INFORMATION 

(deftask "get phone number"
  :doc "Tâche permettant d'obtenir un numéro de téléphone."
  :dialog _get-phone-conversation
  :indexes ("numéro de téléphone" .7 "téléphone" .5 "de" .2 "portable" .7)
  ;("required args" _get-phone-number-data)
  ;("optional args" _get-phone-number-who)
  ;("fail question" "Whose telephone number?")
  )

;;;========================================================================== HELP 

(deftask "help"
  :doc "Tâche (ordre) permettant d'imprimer une information générale."
  :performative :command
  :dialog _print-help-conversation
  :indexes ("au secours" .5 "SOS" .6 "help" .7 "perdu" .4 "perdue" .4
            "que pouvez-vous faire" .4 "quels services" .4)
  )

(deftask "help"
  :doc "Tâche (question) permettant d'imprimer une information générale."
  :dialog _print-help-conversation
  :indexes ("au secours" .5 "SOS" .6 "help" .7 "perdu" .4 "perdue" .4
            "que pouvez-vous faire" .4 "quels services" .4)
  )

;;;=============================================================== LOAD REFERENCES

(deftask "import references"
  :doc "Tâche permettant d'importer un fichier de références au format Bibtex."
  :performative :command
  :dialog _load-references-conversation
  :indexes ("charger" .3 "charge" .3 "importer" .4 "importe" .4 "ouvrir" .3
            "fichier" .3 "biblio" .4 "références" .4 "format" .1 "bibtex" .4)
  )

;;;====================================================== MAKING THEMATIC WEB PAGE

(deftask "make thematic web page"
  :doc "Tâche permettant créer une page web pour des sujets donnés."
  :performative :command
  :dialog _make-thematic-web-page-conversation
  :indexes ("contruire" .1 "créer" .1 "faire" .1 "web" .6 "page" .2 "thématique" .9
            "sujets" .6 "domaines" .6 "thème" .6 "thèmes" .6)
  )

;;;======================================================== MAKING YEARLY WEB PAGE

(deftask "make yearly web page"
  :doc "Tâche permettant créer une page web pour une année donnée."
  :performative :command
  :dialog _make-yearly-web-page-conversation
  :indexes ("contruire" .1 "créer" .1 "faire" .1 "web" .6 "page" .2 "année" .3)
  )

;;;=================================================================== PROCESS ASK 
;;; Generic task to process ASK messages

(deftask "process ask"
  :doc "Tâche permettant de traiter un message de type ASK."
  :performative :command
  :dialog _process-ask-conversation
  :indexes (":process-ask" 1)
  )

;;;================================================================== PROCESS TELL 
;;; Generic task to process TELL messages
;;;   This task displays the content of the message and either keeps it or deletes
;;; it.

(deftask "process tell"
  :doc "Tâche permettant de traiter un message de type TELL."
  :performative :command
  :dialog _process-tell-conversation
  :indexes (":process-tell" 1)
  )

;;;=================================================================== PUBLISH NEW
  
(deftask "publish new"
  :doc  "Tâche permettant publier une nouvelle"
  :performative  :command
  :dialog  _publish-new-conversation
  ;("required args" _publish-new-title _publish-new-content)
  :indexes ("publier" .6 "partager" .3 "externer" .3 "nouvelle" .5
            "s'est passé" .3 "s'est passée" .3 "fait" .3 "événement" .3
            "chose" .2)
  ;("where to ask" :news)
  ;("fail question" "Quelle est le sujet?")
  )

;;;===================================================================== SET FONTS 

(deftask "font handler"
  :doc "Tâche permettant de modifier la taille des caractères sur l'écran."
  :performative :command
  :dialog _set-font-size-conversation
  :indexes ("police" .5 "texte" .3 "lire" .3 "lettres" .5 "écrire" .4 "gros" .2
            "grosse" .2)
  )

;;;========================================================================== TELL

(deftask "tell agent"
  :doc  "Tâche (inform) pour demander une information à un autre agent"
  :performative :command
  :dialog _tell-conversation
  :indexes ("informer" .8 "dire à" .7 "information pour" .7 "prévenir" .7)
  )

;;;========================================================================= TRACE 

(deftask "set trace"
  :doc "Tâche permettant d'activer ou de supprimer la trace."
  :performative :command
  :dialog _trace-conversation
  :indexes ("transition" .5 "transitions" .5 "trace" .5 "verbose" .5
            "bavard" .5)
  )

;;;======================================================================= WHAT-IS 

(deftask "what is"
  :doc "Tâche permettant d'obtenir des infos sur une expression."
  :dialog _what-is-conversation
  :indexes ("qu" .2 "est" .2 "qu est ce" .7 "c est quoi" .7 "qu est-ce" .7)
  )

;;;======================================================================== WHO-IS 

(deftask "who is"
  :doc "Tâche permettant d'obtenir des infos sur une personne."
  :dialog _get-person-info-conversation
  :indexes ("qui" .5 "est" .2 "c est qui" .7 "who" .5 "who is" .5)
  ;:where-to-ask :address
  ;:action :who-is-html
  )

;;;=============================================================================== 

:EOF