﻿;;;-*- Mode: Lisp; Package: "ARS-ADDRESS" -*-
;;;===============================================================================
;;;10/11/21
;;;                               AGENT ARS-ADDRESS 
;;;
;;;===============================================================================

;;; we call our agent ARS-ADDRESS to avoid conflicts with other staff agents of
;;; other PAs

;;; Global behavior
;;; ---------------
;;; Several cases:
;;; - if message does not contain a pattern, then ARS-ADDRESS returns a list of
;;;   possibly 1 string
;;; - if message contains a pattern, then the pattern must be a subset of the
;;;   ARS-ADDRESS ontology. ARS-ADDRESS tries to fill the pattern and returns a
;;;   list of filled patterns

#|
2007
 0831 adding a few functions to select using all words in the input
 0901 shift most functions into MOSS services
 1017 adding :what-is skill
2008
 0605 checking failures on unknown persons modifying get-phone-fill-pattern
2009
 0209 adding master option to agent definition
 0801 renaming agent ARS-ADDRESS. ARS stans for ARISTIDE
2010
 1121 upgrade to OMAS v8.0.8
|#

(defpackage :ARS-ADDRESS (:use :moss :omas :cl #+MCL :ccl))
(in-package :ARS-ADDRESS)

(omas::defagent :ARS-ADDRESS :master (:aristide) :redefine t)

;;;===============================================================================
;;; 
;;;                         service macros and functions 
;;;
;;;===============================================================================


;;; ==================================== globals =================================
;;; Globals  can be used within the agent space to simplify programming, although
;;; it is better to use the agent memory.

(defParameter *address-words* 
  '("address" "home address" "live" "lives"
    "adresse" "domicile" "habite" "vit"))

(defParameter *empty-words* 
  '("the" "a" "an" "of" "from" "to" "those" "these" "that" "s" "?"
    "le" "la" "les" "de" "du" "des" "à" "aux" "ce" "ces" "d" "l"))

(defParameter *phone-words*
  '("phone" "telephone" "phone-number" "telephone-number" "number"
    "téléphone" "numéro" "numéro-de-téléphone"))

(defParameter *default-person-phone-pattern*
  '("personne" ("nom")("prénom")
    ("téléphone" ("téléphone"))))

(defParameter *default-organization-phone-pattern*
  '("organisation" ("nom") ("sigle")
    ("téléphone" ("téléphone"))))

;;;===============================================================================
;;; 
;;;                                skills 
;;;
;;;===============================================================================

;;; ========================== skill section GET ADDRESS =========================

(defskill :GET-address :ARS-ADDRESS
  :static-fcn static-GET-ADDRESS
  )

;;;------------------------------------------------------------ STATIC-GET-ADDRESS

;;; the strategy is to remove the terms associated with addresses and to look for
;;; people, organizations or meetings, namely objects that can have an address
;;; accessing them from entry points.
;;; this will lead to giving the address of UTC instead of the address of the 
;;; president of UTC, since the system reacts on UTC only.
;;;
;;; another approach would be to remove the leading terms containing words like
;;; address, private address, where, live,... to try a formal access with the rest
;;; of the input, and, if that yield results, to try getting an address from the
;;; returned ojects. The problem is that doing so, we loose the type of address
;;; we are looking for and must deal with that in the address dialog, e.g. the 
;;; distinction between professional address and home address.
;;;

(defUn static-get-address (agent message args)
  "We receive a request for an address. We try to obtain the info from the data
Arguments:
   agent: :ARS-ADDRESS
   environment: environment
   args: e.g. ((:data \"adresse\" \"de\" \"Labrousse\" \"?\")
               (:language . :FR)
               (:pattern . (\"personne\" (\"nom\") (\"adresse\"))))
Return:
   :failure or e.g. ((\"personne\" (\"nom\" \"Labrousse\")
                                   (\"adresse\" \"...\")))"
  (declare (ignore message))
  ;; first we should get some entry-point from data
  ;; remove empty words from the data
  (let ((data (cdr (assoc :data args)))
        (*language* (or (cdr (assoc :language args)) *language*))
        (pattern (cdr (assoc :pattern args)))
        input query-list entry-list results)
    ;(format *debug-io* 
    ;    "~&+++ :ARS-ADDRESS static-get-address /data: ~S in package: ~S~
    ;     ~&pattern: ~S" data *package* pattern)
    ;; clean input of parasitic words
    (setq input (moss::%clean-word-list data *empty-words* *address-words*))
    ;(format *debug-io* "~&+++ static-get-address /cleaned input: ~S" input)
    ;; if nothing left, then we can't process anything and return "failure"
    (unless input
      (static-exit agent :failure)
      (return-from static-get-address nil))
    
    ;; try to build formal queries with the rest of input
    (setq query-list (moss::path-build-query-list input))
    ;(format *debug-io* "~&+++ ARS-ADDRESS get-address /query-list:~% ~S" query-list)
    ;; try all queries
    (dolist (query query-list)
      ;; using each query try to access addresses...
      (setq entry-list (access query :already-parsed t))
      ;(format *debug-io* "~&+++ ARS-ADDRESS get-address /entry-list:~% ~S" entry-list)
      ;; we exit on the first resuls we get (why?)
      (setq results (append results entry-list)))
    
    (setq results (delete-duplicates results))
    
    ;; if there is no pattern, we return a string containing name, first-name and
    ;; full address
    ;; if there is a pattern we fill the pattern
  
      (setq results (fill-pattern results pattern :default-method '=get-address))
      ;; otherwise, compute a single address string
      ;(setq results (get-addresses results pattern)))
    ;(format *debug-io* "~&+++ results: ~S" results)
    
    ;; return the answer
    (static-exit agent (or results :failure))))

#|
? (static-get-address SA_ARS-ADDRESS nil '((:data "adresse" "de" "Dominique" "?")))
|#
;;;------------------------------------------------------------------- GET-ADDRESS

(defUn get-addresses (object-list &optional pattern)
  "accesses the local database using the computed access list ~
   e.g. ($E-PERSON.1  $E-PERSON.2)."
  ;; for the time being we access the returned objects directly
  (let (results)
    ;; debug check object-list
    ;(format *debug-io* "~&+++ get-addresses object-list: ~&~S" object-list)
    
    (dolist (obj-id (delete-duplicates object-list))
      ;; check for address link
      (when (moss::send obj-id 'moss::=get 'HAS-ADDRESS)
        ;; if pattern, fill pattern, otherwise send everything
        (if pattern
          (push (moss::fill-answer-pattern-for-one-level pattern obj-id) results)
          ;; otherwise use the method returning strings
          (push (send obj-id 'moss::=make-print-string) results)
          )))
    (reverse results)))

#|
? (get-addresses '($E-PERSON.1 $E-PERSON.2 $E-PERSON.3))
(("person" ("name" "Barthes") ("first-name" "Jean-Paul")
  ("phone" ("home phone" ("number" "(0)3 44 23 31 37"))
   ("office phone" ("number" "(0)3 44 23 44 66"))
   ("cell phone" ("number" "(0)6 80 45 32 67")))
  ("address"
   ("home address" ("street and number" "14, Allée de la Tilloye")
    ("town" "Compiègne") ("country" "France") ("zip" "60200")))
  ("email" ("email address" ("code" "barthes@utc.fr")))
  ("web page" ("web address" ("URL" "http://www.utc.fr/~barthes"))))
 ("person" ("name" "Barthes Biesel" "Barthes" "Biesel") ("first-name" "Dominique")
  ("phone" ("home phone" ("number" "(0)3 44 23 31 37"))
   ("office phone" ("number" "(0)3 44 23 43 93"))
   ("cell phone" ("number" "(0)6 81 24 46 66")))
  ("address"
   ("home address" ("street and number" "14, Allée de la Tilloye")
    ("town" "Compiègne") ("country" "France") ("zip" "60200")))
  ("email" ("email address" ("code" "dbb@utc.fr")))))

? (get-addresses '($E-PERSON.1 $E-PERSON.2 $E-PERSON.3)
                 '("person" 
                   ("name")
                   ("first-name")
                   ("address" 
                    ("address" ("street") ("zip") ("town")))))
(("person" ("name" "Barthes") ("first-name" "Jean-Paul")
  ("address"
   ("home address" ("street" "14, Allée de la Tilloye") ("zip" "60200")
    ("town" "Compiègne"))))
 ("person" ("name" "Barthes Biesel" "Barthes" "Biesel") ("first-name" "Dominique")
  ("address"
   ("home address" ("street" "14, Allée de la Tilloye") ("zip" "60200")
    ("town" "Compiègne")))))
|#
;;; ======================= skill section GET PHONE NUMBER =======================

;;; Strategy to get a phone number.
;;; We get a list of words or sentence that can be:
;;; 1. - numéro de téléphone de barthès?
;;; 2. - phone Dominique?
;;;or, in French
;;; 3. - téléphone du président de l'UTC
;;; 4. - téléphone du CNRS?
;;;
;;; In général the mention of phone is the goal of the request and starts the sentence.
;;; If we plug the sentence directly into MOSS, then we'll obtain:
;;; 1. ($E-PHONE ($S-PERSON-PHONE.OF (> 0) ($E-PERSON ($T-PERSON-NAME :IS "Barthès"))))
;;; 2. ($E-PHONE
;;;     ($S-PERSON-PHONE.OF (> 0) ($E-PERSON ($T-PERSON-FIRST-NAME :IS "Dominique"))))
;;; 3. ($E-PHONE
;;;     ($S-PERSON-PHONE.OF (> 0)
;;;      ($E-PERSON
;;;       ($S-TEACHING-ORGANIZATION-PRESIDENT.OF (> 0) ($E-TEACHING-ORGANIZATION #)))))
;;; 4. ($E-PHONE
;;;     ($S-ORGANIZATION-PHONE.OF (> 0)
;;;      ($E-ORGANIZATION ($T-ORGANIZATION-ACRONYM :IS "UTC"))))
;;;
;;; Now if we access the KB  using this query we get:
;;; 1. ($E-OFFICE-PHONE.1 $E-CELL-PHONE.1 $E-HOME-PHONE.1 $E-OFFICE-PHONE.2 
;;;     $E-CELL-PHONE.2 $E-HOME-PHONE.2 $E-CELL-PHONE.3)
;;;   i.e. all the telephone number of all the barthès
;;; 2. ($E-HOME-PHONE.1 $E-OFFICE-PHONE.2 $E-CELL-PHONE.2)
;;; 3. ($E-CELL-PHONE.5)
;;; 4. ($E-OFFICE-PHONE.4)
;;;
;;; We need to provide more information when there is more than one answer, first
;;; distinguishing among people and organizations.

(defskill :GET-PHONE-NUMBER :ARS-ADDRESS
  :static-fcn static-GET-PHONE-NUMBER
  )

(defUn static-get-phone-number (agent environment args)
  "We receive a request for a phone number. We try to obtain the info from the ~
   data.
Arguments:
   agent: :ARS-ADDRESS
   environment: environment
   args: alist e.g. ((:data \"of\" \"Barthes\") (:pattern ...))
Return:
   :failure or filled pattern."
  (declare (ignore environment))
  ;; first we should get some entry-point from data
  (let ((data (cdr (assoc :data args)))
        (pattern (cdr (assoc :pattern args)))
        (*language* (or  (cdr (assoc :language args)) *language*))
        query-list results entry-list)
    ;(format *debug-io* 
    ;    "~&+++ :ARS-ADDRESS static-get-phone-number /data: ~S package: ~S~
    ;     ~&pattern: ~S language ~S" data *package* pattern *language*)
    ;; access entities by trying to build a formal query
    (setq query-list (moss::path-build-query-list data))
    ;(format *debug-io* "~&+++ ARS-ADDRESS static-get-phone-number/ query-list:~% ~S" 
    ;  query-list)
    ;; try all answers
    (dolist (query query-list)
      ;; we want to avoid returning ALL telephone numbers when the owner has not been
      ;; identified
      (when (cdr query)
        ;; then using the query try to access phones...
        (setq entry-list (access query :already-parsed t))
        ;(format *debug-io* "~&+++ ARS-ADDRESS static-get-phone-number/ results:~% ~S" 
        ;  results)
        (setq results (append results entry-list))))
    ;(format *debug-io* "~&+++ ARS-ADDRESS static-get-phone-number/ collected results:~% ~S" 
    ;  results)
    ;; if we have results then we won, put the result into a nice format
    (setq results
          (if results (get-phone-fill-pattern (delete-duplicates results) pattern)))
    ;(format *debug-io* "~&+++ ARS-ADDRESS static-get-phone-number/ final results:~% ~S" 
    ;  results)
    ;; return the answer
    (static-exit agent (or results "*failure*"))))

;;;-------------------------------------------------------- GET-PHONE-FILL-PATTERN
;;; From the phone numbers must produce something like:
;;;  ("téléphone du domicile" "0344303137"
;;;     ("personne" ("nom" "Barthes") ("prénom" "Jean-Paul"))
;;;     ("personne" ("nom" "Barthes" "Biesel") ("prénom" "Dominique")))
;;; so that we can print:
;;;  0344303137 (domicile: Jean-Paul Barthès, Dominique Barthès Biesel)

(defUn get-phone-fill-pattern (object-list &optional pattern)
  "accesses the local database using the computed access list ~
   e.g. ($E-HOME-PHONE.1 $E-OFFICE-PHONE.2).
   Returns only info for any object having a phone property.
Arguments:
   access-list: list of objects representing telephone numbers
   pattern (opt): pattern to fill with data from objects
Return:
   a tree structure of strings reflecting the MOSS structure."
  (declare (ignore pattern))
  (let (owners results entry description)
    ;; debug: check object-list
    ;(format *debug-io* "~&+++ get-phone-fill-pattern /object-list: ~&  ~S"
    ;        object-list)
    ;; for each phone, get the owner
    (dolist (phone object-list)
      ;; first get the data concerning the number
      (setq entry 
            (moss::fill-answer-pattern-for-one-level 
             (case *language*
               (:fr '("téléphone"))
               (:en '("phone")))
             phone))
      ;(print `(+++entry ,entry))
      (when entry
        ;; then get owners
        (setq owners (send phone '=get 'IS-PHONE-OF))
        ;; for each owner get the relevant info
        (dolist (owner owners)
          (setq description
                (append
                 ;; get type
                 (send (car (moss::%type-of owner)) '=summary)
                 ;; get summary
                 (list (send owner '=summary))))
          ;; update entry
          (setq entry (append entry (list description))))
        (push entry results)))
    (reverse results)))

#|
(get-phone-fill-pattern '($E-HOME-PHONE.1))
(("téléphone du domicile" ("+33 (0)3 44 23 31 37")
  ("personne" ("Barthès: Jean-Paul"))
  ("personne" ("Barthès, Barthès-Biesel: Dominique"))))

(get-phone-fill-pattern '($E-OFFICE-PHONE.1 $E-CELL-PHONE.1 $E-HOME-PHONE.1 $E-OFFICE-PHONE.2 
     $E-CELL-PHONE.2 $E-HOME-PHONE.2 $E-CELL-PHONE.3))
(("téléphone professionnel" ("+33 (0)3 44 23 44 66")
  ("personne" ("Barthès: Jean-Paul")))
 ("portable" ("+33 (0)6 80 45 32 67") ("personne" ("Barthès: Jean-Paul")))
 ("téléphone du domicile" ("+33 (0)3 44 23 31 37")
  ("personne" ("Barthès: Jean-Paul"))
  ("personne" ("Barthès, Barthès-Biesel: Dominique")))
 ("téléphone professionnel" ("+33 (0)3 44 23 43 93")
  ("personne" ("Barthès, Barthès-Biesel: Dominique")))
 ("portable" ("+33 (0)6 81 24 46 66")
  ("personne" ("Barthès, Barthès-Biesel: Dominique")))
 ("téléphone du domicile" ("+86 571 8701 0782") ("personne" ("Barthès: Camille")))
 ("portable" ("+86 134 5678 3173") ("personne" ("Barthès: Camille"))))

? (get-phone-fill-pattern '($E-OFFICE-PHONE.4))
(("téléphone professionnel" ("+33 3 44 32 44 23") ("université" ("UTC "))))
|#
;;; =========================== skill section :WHO-IS ============================

(defskill :WHO-IS :ARS-ADDRESS
  :static-fcn static-WHO-IS
  )

(defUn static-who-is (agent environment args)
  "We receive a request for info on a person.
Arguments:
   agent: :ARS-ADDRESS
   environment: environment
   args: e.g. ((:data \"of\" \"Barthes\"))
Return:
   :failure or e.g. (:address \"03 44 20 31 37\")."
  (declare (ignore environment))
  ;; first we should get some entry-point from data
  ;; remove empty words from the data
  (let ((data (cdr (assoc :data args)))
        (*language* (or (cdr (assoc :language args)) *language*))
        query-list person-list results)
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHO-IS /data: ~S in package: ~S"
    ;  data *package*)
    (unless data
      (return-from static-who-is (static-exit agent "*failure*")))
    ;; try to obtain a formal query from the ontology and the KB
    (setq query-list (moss::path-build-query-list data))
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHO-IS /query-list: ~S" query-list)
    ;; if nothing, do not need to answer
    (unless query-list
      (return-from static-who-is (static-exit agent "*failure*")))
    ;;  otherwise apply all queries merging results
    (dolist (query query-list)
      (setq person-list (append person-list (access query :already-parsed t))))
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHO-IS /person-list: ~S" person-list)
    ;; if we got something here, we should access the publications, counts them,
    (setq results 
          (broadcast 
           (delete-duplicates person-list)
           '=get-documentation :no-summary t))
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHO-IS /results: ~S" results)
    (static-exit agent (or results "*failure*"))))

;;;============================ skill section WHAT-IS ============================

(defskill :WHAT-IS :ARS-ADDRESS
  :static-fcn static-WHAT-IS
  )

(defUn static-what-is (agent environment args)
  "We receive a request for info on a person.
Arguments:
   agent: :ARS-ADDRESS
   environment: environment
   args: e.g. ((:data \"of\" \"Barthes\"))
Return:
   :failure or e.g. (\"person\" (\"name\" \"barthes\"))."
  (declare (ignore environment))
  ;; first we should get some entry-point from data
  ;; remove empty words from the data
  (let ((data (cdr (assoc :data args)))
        (*language* (or (cdr (assoc :language args)) *language*))
        query-list object-list results)
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHAT-IS /data: ~S in package: ~S"
    ;  data *package*)
    (unless data
      (return-from static-WHAT-IS (static-exit agent "*failure*")))
    ;; try to obtain a formal query from the ontology and the KB
    (setq query-list (moss::path-build-query-list data))
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHAT-IS /query-list: ~S" query-list)
    ;; if nothing, do not need to answer
    (unless query-list
      (return-from static-WHAT-IS :abort))
    ;;  otherwise apply all queries merging results
    (dolist (query query-list)
      (setq object-list (append object-list (access query :already-parsed t))))
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHAT-IS /object-list: ~S" object-list)
    ;; if we got something here, we should access the publications, counts them,
    (setq results 
          (broadcast 
           (delete-duplicates object-list)
           '=get-definition-summary))
    ;(format *debug-io* "~&+++ ARS-ADDRESS WHAT-IS /results: ~S" results)
    (static-exit agent (or results "*failure*"))))

#|
;;;============================== skill section =========================

(defParameter *ADDRESS-xxx-functions*
  '((:functions static-XXX 
                dynamic-XXX)))

(defskill :XXX ADDRESS
  :static-fcn static-XXX
  :dynamic-fcn dynamic-XXX
  :time-limit-fcn time-limit-XXX
  :timeout-handler timeout-handler-XXX
  :preconditions preconditions-XXX
  :select-best-answer-fcn select-best-answer-XXX
  :acknowledge-fcn acknowledge-XXX
  )

(defun static-xxx (agent environment <args>)
  "documentation"
  (static-exit agent <result>))

(defun dynamic-xxx (agent environment <args>)
  "documentation"
  (dynamic-exit agent <result>))

(defun time-limit-xxx (agent environment message)
  "documentation"
  )

(defun timeout-handler-xxx (agent message)
  "documentation"
  )

(defun preconditions-xxx (agent environment <args>)
  "documentation"
  )

(defun select-best-answer-xxx (agent environment answer-message-list)
  "documentation"
  )

(defun acknnowledge-xxx (agent environment<args>)
  "documentation"
  )
|#
;;; ================================= skill section ==============================

(defskill :hello :ARS-ADDRESS
  :static-fcn static-hello
  )

(defUn static-hello (self environment)
  "greeting function"
  (declare (ignore environment))
  (static-exit self "Hello there..."))

;;;===============================================================================
;;; 
;;;                               initial conditions 
;;;
;;;===============================================================================

:ENV

;;;==============================================================================
;;; 
;;;                                    goals
;;;
;;;==============================================================================

;;; ================================= goal section ==============================

#|
"macro that builds a goal structure for an agent.
Arguments:
   goal-name:              name of the goal that will be used to create task-ids
   agent-name:             name of the agent to receive the goal
keys arguments:
   mode:                   activation mode (:rigid or :flexible)
   type:                   type of goal (:permanent :cyclic :1-shot)
   period:                 period for cyclic goals (default is 20s)
   expiration-date:        date at which the goal dies
   expiration-delay:       time to wait until we kill the goal
   importance:             on a 1-100 scale (not yet available)
   urgency:                on a 1-100 scale (not yet available)
   activation-date:        date at which the goal should fire (default is now)
   activation-delay:       time to wait before activating the goal
   activation-level:       on a 1-100 scale (default 50)  (not yet available))
   activation-threshold:   on a 1-100 scale (default 50)  (not yet available)
   activation-change-fcn:  optional function called at each cycle (not yet available)
   status:                 waiting, active, dead,... 
                              may not be useful with activation
   goal-enable-fcn:        function checking the goal enabling conditions
   script:                 script describing the goal, currently a function that
                           returns a list of messages
Return:
   goal-name."

;;; return T to enable the goal NIL ti inhibit it

(defun enable-ggg (agent script)
  "documentation"
  t)

(defun goal-ggg-ADDRESS ()
  (list (make-instance 'omas-message :type :request
                       :from :ARS-ADDRESS :to :ARS-ADDRESS
                       :action
                       :args
                       :task-id :GGG
                       )))

;;; as soon as defgoal is executed goal is launched

(defgoal GGG :ARS-ADDRESS
  mode:           
 type: 
  period: 
  expiration-date: 
 expiration-delay: 
  activation-date: 
  activation-delay: 
  status: 
  goal-enable-fcn: 
  script: goal-ggg-ADDRESS
  )
|#


:EOF